/**
 * 
 */
package com.wieseke.cptk.common.dao;

import java.util.ArrayList;
import java.util.List;

import com.wieseke.cptk.common.api.IPositionNode;

/**
 * @author Nicolas Wieseke
 * 
 */
public abstract class PositionNode implements IPositionNode {

	/**
	 * the number of the node.
	 */
	private int number;

	/**
	 * the node label.
	 */
	private String label;

	/**
	 * the node level.
	 */
	private int level;

	/**
	 * the parent node.
	 */
	protected IPositionNode parent;

	/**
	 * the array list of child nodes.
	 */
	protected List<IPositionNode> children;

	/**
	 * the number of all descending nodes in the subtree starting at this node.
	 */
	private int childrenCount;

	/**
	 * the x position of the node
	 */
	private int posX;
	/**
	 * the x position of the node
	 */
	private int posY;

	/**
	 * default constructor
	 */
	public PositionNode() {
		super();
		this.number = -1;
		this.label = "";
		this.parent = null;
		this.children = new ArrayList<IPositionNode>();
		this.childrenCount = 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.INode#getNumber()
	 */
	public int getNumber() {
		return number;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.INode#setNumber(int)
	 */
	public void setNumber(int number) {
		this.number = number;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.INode#getLabel()
	 */
	public String getLabel() {
		return label;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.INode#setLabel(java.lang.String)
	 */
	public void setLabel(String label) {
		this.label = label;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.INode#getLevel()
	 */
	
	public int getLevel() {
		return this.level;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.INode#setLevel(int)
	 */
	
	public void setLevel(int level) {
		this.level = level;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IPositionNode#getParent()
	 */
	public IPositionNode getParent() {
		return parent;
	}

	/**
	 * @param parent
	 *            the parent to set
	 */
	public void setParent(IPositionNode parent) {
		this.parent = parent;
		if (parent != null)
			this.level = parent.getLevel() + 1;
		else
			this.level = 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IPositionNode#getChildren()
	 */
	public List<? extends IPositionNode> getChildren() {
		return children;
	}

	/**
	 * @param children
	 *            the children to set
	 */
	public void setChildren(List<IPositionNode> children) {
		this.children = children;
	}

	/**
	 * @param child
	 * @return the result of the lists add method
	 */
	public boolean addChild(IPositionNode child) {
		return children.add(child);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.INode#getDirectChildrenCount()
	 */
	public int getDirectChildrenCount() {
		return this.children == null ? 0 : this.children.size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.INode#getChildrenCount()
	 */
	
	public int getChildrenCount() {
		return childrenCount;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.INode#setChildrenCount(int)
	 */
	
	public void setChildrenCount(int childrenCount) {
		this.childrenCount = childrenCount;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.INode#isLeaf()
	 */
	
	public boolean isLeaf() {
		return this.children == null ? true : this.children.size() == 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IPositionNode#getPosX()
	 */
	
	public int getPosX() {
		return posX;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IPositionNode#setPosX(int)
	 */
	
	public void setPosX(int posX) {
		this.posX = posX;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IPositionNode#getPosY()
	 */
	
	public int getPosY() {
		return posY;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IPositionNode#setPosY(int)
	 */
	
	public void setPosY(int posY) {
		this.posY = posY;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	
	public String toString() {
		return this.getNumber() + ":" + this.getLabel();
	}

}
