/**
 * 
 */
package com.wieseke.cptk.common.provider;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.Map.Entry;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.Viewer;

import com.wieseke.cptk.common.model.PropertiesElement;
import com.wieseke.cptk.input.model.InputCophylogenyProperties;
import com.wieseke.cptk.output.model.OutputCophylogenyProperties;

/**
 * @author Nic
 * 
 */
public class CophylogenyPropertiesContentProvider implements
		IStructuredContentProvider, PropertyChangeListener {

	private final Viewer viewer;

	public CophylogenyPropertiesContentProvider(Viewer viewer) {
		this.viewer = viewer;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java
	 * .lang.Object)
	 */
	
	public Object[] getElements(Object inputElement) {
		if (inputElement instanceof InputCophylogenyProperties) {
			InputCophylogenyProperties cophylogenyInfo = (InputCophylogenyProperties) inputElement;
			PropertiesElement[] content = new PropertiesElement[cophylogenyInfo
					.getOptions().size() + 5];
			content[0] = new PropertiesElement("filename", cophylogenyInfo
					.getFilename());
			content[1] = new PropertiesElement("cospeciation cost",
					cophylogenyInfo.getCospeciationCosts());
			content[2] = new PropertiesElement("sorting cost", cophylogenyInfo
					.getSortingCosts());
			content[3] = new PropertiesElement("duplication cost",
					cophylogenyInfo.getDuplicationCosts());
			content[4] = new PropertiesElement("hostswitch cost",
					cophylogenyInfo.getHostswitchCosts());
			int index = 5;
			Iterator<Entry<String, String>> iterator = cophylogenyInfo
					.getOptions().iterator();
			while (iterator.hasNext()) {
				Entry<String, String> entry = iterator.next();
				content[index] = new PropertiesElement(entry.getKey()
						.toLowerCase(), entry.getValue());
				index++;
			}
			return content;
		}
		if (inputElement instanceof OutputCophylogenyProperties) {
			OutputCophylogenyProperties cophylogenyInfo = (OutputCophylogenyProperties) inputElement;
			PropertiesElement[] content = new PropertiesElement[cophylogenyInfo
					.getOptions().size() + 13];
			content[0] = new PropertiesElement("filename", cophylogenyInfo
					.getFilename());
			content[1] = new PropertiesElement("quality", cophylogenyInfo
					.getSolution().getQuality().toPlainString());
			content[2] = new PropertiesElement("root mapping", cophylogenyInfo
					.getRootMapping());
			content[3] = new PropertiesElement("valid", cophylogenyInfo
					.getValid());
			content[4] = new PropertiesElement("cospeciations", String
					.valueOf(cophylogenyInfo.getSolution().getEvents()
							.getCospeciations()));
			content[5] = new PropertiesElement("sortings", String
					.valueOf(cophylogenyInfo.getSolution().getEvents()
							.getSortings()));
			content[6] = new PropertiesElement("duplications", String
					.valueOf(cophylogenyInfo.getSolution().getEvents()
							.getDuplications()));
			content[7] = new PropertiesElement("hostswitchs", String
					.valueOf(cophylogenyInfo.getSolution().getEvents()
							.getHostswitches()));
			content[8] = new PropertiesElement("total costs", cophylogenyInfo
					.getSolution().getTotalCosts().toPlainString());
			content[9] = new PropertiesElement("cospeciation cost",
					cophylogenyInfo.getCospeciationCosts());
			content[10] = new PropertiesElement("sorting cost", cophylogenyInfo
					.getSortingCosts());
			content[11] = new PropertiesElement("duplication cost",
					cophylogenyInfo.getDuplicationCosts());
			content[12] = new PropertiesElement("hostswitch cost",
					cophylogenyInfo.getHostswitchCosts());
			int index = 13;
			Iterator<Entry<String, String>> iterator = cophylogenyInfo
					.getOptions().iterator();
			while (iterator.hasNext()) {
				Entry<String, String> entry = iterator.next();
				content[index] = new PropertiesElement(entry.getKey()
						.toLowerCase(), entry.getValue());
				index++;
			}
			return content;
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	
	public void dispose() {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface
	 * .viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

	}

	
	public void propertyChange(PropertyChangeEvent arg0) {
		this.viewer.refresh();
	}

}
