/**
 * 
 */
package com.wieseke.cptk.common.view;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import com.wieseke.cptk.common.provider.CophylogenyPropertiesContentProvider;
import com.wieseke.cptk.common.provider.CophylogenyPropertiesLabelProvider;
import com.wieseke.cptk.common.selection.CophylogenyViewerSelection;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.editor.MultiPageInputEditor;
import com.wieseke.cptk.input.editor.VisualInputEditor;
import com.wieseke.cptk.input.model.InputCophylogenyProperties;
import com.wieseke.cptk.output.dao.OutputCophylogeny;
import com.wieseke.cptk.output.editor.VisualOutputEditor;
import com.wieseke.cptk.output.model.OutputCophylogenyProperties;

/**
 * @author Nic
 * 
 */
public class CophylogenyPropertiesView extends ViewPart {

	public static final String ID = "com.wieseke.cptk.common.view.CophylogenyPropertiesView";

	private TableViewer viewer;
	private TableColumn nameColumn;
	private TableColumn valueColumn;

	private ISelectionListener cophylogenySelectionListener;

	/**
	 * 
	 */
	public CophylogenyPropertiesView() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	
	public void createPartControl(Composite parent) {
		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION);
		final Table table = viewer.getTable();

		this.nameColumn = new TableColumn(table, SWT.LEFT);
		this.nameColumn.setText("Property");
		this.nameColumn.setWidth(150);

		this.valueColumn = new TableColumn(table, SWT.LEFT);
		this.valueColumn.setText("Value");
		this.valueColumn.setWidth(300);

		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		viewer.setContentProvider(new CophylogenyPropertiesContentProvider(
				viewer));
		viewer.setLabelProvider(new CophylogenyPropertiesLabelProvider());
		// getSite().setSelectionProvider(viewer);
		hookCophylogenySelection();
	}

	private void hookCophylogenySelection() {
		this.cophylogenySelectionListener = new ISelectionListener() {
			/*
			 * (non-Javadoc)
			 * 
			 * @see
			 * org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse
			 * .ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
			 */
			
			public void selectionChanged(IWorkbenchPart part,
					ISelection selection) {
				cophylogenySelectionChanged(part, selection);

			}
		};
		getSite().getPage().addSelectionListener(
				this.cophylogenySelectionListener);
	}

	protected void cophylogenySelectionChanged(IWorkbenchPart part,
			ISelection selection) {
		if (part == this)
			return;
		if (!(part instanceof MultiPageInputEditor || part instanceof VisualOutputEditor)) {
			this.viewer.setInput(null);
			return;
		} else if ((part instanceof MultiPageInputEditor)
				&& !(((MultiPageInputEditor) part).getSelectedPage() instanceof VisualInputEditor)) {
			this.viewer.setInput(null);
			return;
		}
		if (!selection.isEmpty()
				&& selection instanceof CophylogenyViewerSelection) {
			CophylogenyViewerSelection cophylogenySelection = (CophylogenyViewerSelection) selection;
			if (cophylogenySelection.getCophylogenyViewer().getCophylogeny() instanceof InputCophylogeny) {
				InputCophylogenyProperties cophylogenyInfo = new InputCophylogenyProperties(
						(InputCophylogeny) cophylogenySelection
								.getCophylogenyViewer().getCophylogeny());
				this.viewer.setInput(cophylogenyInfo);
			} else if (cophylogenySelection.getCophylogenyViewer()
					.getCophylogeny() instanceof OutputCophylogeny) {
				OutputCophylogenyProperties cophylogenyInfo = new OutputCophylogenyProperties(
						(OutputCophylogeny) cophylogenySelection
								.getCophylogenyViewer().getCophylogeny());
				this.viewer.setInput(cophylogenyInfo);
			}
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	
	public void dispose() {
		if (this.cophylogenySelectionListener != null) {
			getSite().getPage().removeSelectionListener(
					this.cophylogenySelectionListener);
		}
		super.dispose();
	}
}
