package com.wieseke.cptk.common.wizards;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.common.viewer.CophylogenyViewer;

/**
 * This is a sample new wizard. Its role is to create a new file resource in the
 * provided container. If the container resource (a folder or a project) is
 * selected in the workspace when the wizard is opened, it will accept it as the
 * target container. The wizard creates one file with the extension "nex". If a
 * sample multi-page editor (also available as a template) is registered for the
 * same extension, it will be able to open it.
 */

public class ExportSvgFileWizard extends Wizard {
	private ExportSvgFileWizardPage page;
	private CophylogenyViewer viewer;

	/**
	 * Constructor for NewNexusFileWizard.
	 */
	public ExportSvgFileWizard(CophylogenyViewer viewer) {
		super();
		this.viewer = viewer;
		setNeedsProgressMonitor(true);
	}

	/**
	 * Adding the page to the wizard.
	 */

	public void addPages() {
		IStructuredSelection currentSelection = (IStructuredSelection) PlatformUI
				.getWorkbench().getActiveWorkbenchWindow()
				.getSelectionService().getSelection();
		page = new ExportSvgFileWizardPage(this.viewer, currentSelection);
		addPage(page);
	}

	/**
	 * This method is called when 'Finish' button is pressed in the wizard. We
	 * will create an operation and run it using wizard as execution context.
	 */
	public boolean performFinish() {
		page.createNewFile();
		return true;
	}

}