/**
 * 
 */
package com.wieseke.cptk.input.operation;

import java.util.List;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.wieseke.cptk.input.dao.InputHostNode;
import com.wieseke.cptk.input.dao.InputParasiteNode;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;
import com.wieseke.cptk.input.viewer.InputTreeArrangement;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditAddHostNodeOperation extends AbstractOperation {

	private InputCophylogenyViewer viewer;
	private InputHostNode node;
	private InputHostNode newNode;
	private List<InputParasiteNode> oldAssociations;
	InputTreeArrangement arrangement;

	public EditAddHostNodeOperation(InputCophylogenyViewer viewer, InputHostNode node) {
		super("add child to node " + node.getLabel());
		this.viewer = viewer;
		this.node = node;
		this.oldAssociations = viewer.getCophylogeny().getAssociationPartners(node,
				viewer.getCophylogeny().getParasiteRoot());
		this.arrangement = new InputTreeArrangement(this.viewer.getCophylogeny());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		InputHostNode child = new InputHostNode();
		EditChangeLabelOperation operation = new EditChangeLabelOperation(this.viewer, child);
		try {
			IStatus status = operation.execute(null, null);
			if (status.equals(Status.OK_STATUS)) {
				child.setParent(this.node);
				child.setRank(this.node.getRank());
				this.node.getChildren().add(child);
				this.newNode = child;
				for (InputParasiteNode parasiteNode : this.oldAssociations) {
					parasiteNode.getAssociations().remove(this.node);
				}
				this.viewer.initCophylogeny(true, this.arrangement);
				this.viewer.getCophylogeny().recountAndRenumber();
				this.viewer.setDirty(true);
			}
		} catch (ExecutionException e) {
			return Status.CANCEL_STATUS;
		}
		return Status.OK_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		this.node.getChildren().add(this.newNode);
		for (InputParasiteNode parasiteNode : this.oldAssociations) {
			parasiteNode.getAssociations().remove(this.node);
		}
		this.viewer.initCophylogeny(true, this.arrangement);
		this.viewer.getCophylogeny().recountAndRenumber();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		this.node.getChildren().remove(newNode);
		for (InputParasiteNode parasiteNode : this.oldAssociations) {
			parasiteNode.getAssociations().add(this.node);
		}
		this.viewer.initCophylogeny(true, this.arrangement);
		this.viewer.getCophylogeny().recountAndRenumber();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

}
