/**
 * 
 */
package com.wieseke.cptk.input.viewer;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.graphics.Transform;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.common.api.IHostNode;
import com.wieseke.cptk.common.api.IPositionHostNode;
import com.wieseke.cptk.common.api.IPositionParasiteNode;
import com.wieseke.cptk.common.api.ITreeArrangement;
import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.common.viewer.CophylogenyViewer;
import com.wieseke.cptk.common.viewer.NodeControl;
import com.wieseke.cptk.input.constants.InputViewerConstants;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.preferences.InputPreferenceConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class InputCophylogenyViewer extends CophylogenyViewer {
	/**
	 * Logger
	 */
	static Logger logger = Logger.getLogger(InputCophylogenyViewer.class);

	private int treeMode;
	private int editMode;

	private List<InputHostParasiteNodeControlAssociation> associations;

	private InputNodeControl phiSource;
	private Point phiTargetPosition;
	private boolean phiAssociationExists;
	private String linetype;
	/**
	 * the width of the association lines
	 */
	protected int associationLineWidth;
	/**
	 * the color of the association lines
	 */
	protected Color associationLineColor;

	/**
	 * default constructor
	 * 
	 * @param parent
	 * @param style
	 * @param cophylogeny
	 */
	public InputCophylogenyViewer(Composite parent,
			InputCophylogeny cophylogeny, String name) {
		super(parent, cophylogeny, name);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.viewer.CophylogenyViewer#init()
	 */
	protected void init() {

		IPreferenceStore store = PlatformUI.getPreferenceStore();
		this.hostLineWidth = store
				.getInt(InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_LINEWIDTH);
		this.parasiteLineWidth = store
				.getInt(InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_LINEWIDTH);
		this.hostLineColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_LINECOLOR));
		this.parasiteLineColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_LINECOLOR));
		this.hostFont = new Font(Display.getDefault(), PreferenceConverter
				.getFontData(store,
						InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_FONT));
		this.parasiteFont = new Font(
				Display.getDefault(),
				PreferenceConverter
						.getFontData(
								store,
								InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_FONT));
		this.hostFontColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_FONTCOLOR));
		this.parasiteFontColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_FONTCOLOR));
		this.linetype = store
				.getString(InputPreferenceConstants.INPUT_VIEWER_NAME_LINETYPE);
		this.associationLineWidth = store
				.getInt(InputPreferenceConstants.INPUT_VIEWER_NAME_ASSOCIATION_LINEWIDTH);
		this.associationLineColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								InputPreferenceConstants.INPUT_VIEWER_NAME_ASSOCIATION_LINECOLOR));

		this.associations = new ArrayList<InputHostParasiteNodeControlAssociation>();

		this.phiSource = null;
		this.phiTargetPosition = null;
		this.phiAssociationExists = false;

		this.treeMode = InputViewerConstants.TREE_SELECTION_BOTH;
		this.editMode = InputViewerConstants.EDIT_MODE_FOLD_NODE;

		this.container.addPaintListener(new PaintListener() {

			public void paintControl(PaintEvent e) {
				drawNodeLines(InputViewerConstants.TREE_SELECTION_HOST, e.gc);
				drawNodeLines(InputViewerConstants.TREE_SELECTION_PARASITE,
						e.gc);
				drawAssociationLines(e.gc);
				if (InputCophylogenyViewer.this.showHostLabels) {
					drawLabels(InputViewerConstants.TREE_SELECTION_HOST, e.gc);
				}
				if (InputCophylogenyViewer.this.showParasiteLabels) {
					drawLabels(InputViewerConstants.TREE_SELECTION_PARASITE,
							e.gc);
				}
			}
		});
		this.container.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
			}

			public void mouseDown(MouseEvent e) {
			}

			public void mouseUp(MouseEvent e) {
				if (e.button == 3) {
					phiSource = null;
					container.redraw();
				}

			}
		});
		this.container.addMouseMoveListener(new MouseMoveListener() {

			public void mouseMove(MouseEvent e) {
				if (phiSource != null) {
					phiTargetPosition = new Point(e.x, e.y);
					container.redraw();
				}
			}
		});

		initCophylogeny(true, null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.wieseke.cptk.common.viewer.CophylogenyViewer#updatePreferences(org
	 * .eclipse.jface.util.PropertyChangeEvent)
	 */

	protected void updatePreferences(PropertyChangeEvent event) {
		IPreferenceStore store = PlatformUI.getPreferenceStore();
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_LINETYPE)) {
			this.linetype = (String) event.getNewValue();
			return;
		}
		if (event
				.getProperty()
				.equals(
						InputPreferenceConstants.INPUT_VIEWER_NAME_ASSOCIATION_LINEWIDTH)) {
			this.associationLineWidth = (Integer) event.getNewValue();
			return;
		}
		if (event
				.getProperty()
				.equals(
						InputPreferenceConstants.INPUT_VIEWER_NAME_ASSOCIATION_LINECOLOR)) {
			this.associationLineColor = new Color(
					getDisplay(),
					PreferenceConverter
							.getColor(
									store,
									InputPreferenceConstants.INPUT_VIEWER_NAME_ASSOCIATION_LINECOLOR));
			return;
		}

		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_NODEWIDTH)) {
			for (NodeControl node : hostNodesControls) {
				((InputNodeControl) node).setWidth((Integer) event
						.getNewValue());
			}
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_NODEHEIGHT)) {
			for (NodeControl node : hostNodesControls) {
				((InputNodeControl) node).setHeight((Integer) event
						.getNewValue());
			}
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_NODECOLOR)) {
			for (NodeControl node : hostNodesControls) {
				((InputNodeControl) node)
						.setNodeColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_NODECOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(
						InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_NODEOUTLINECOLOR)) {
			for (NodeControl node : hostNodesControls) {
				((InputNodeControl) node)
						.setNodeOutlineColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_NODEOUTLINECOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(
						InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_NODEHIGHLIGHTCOLOR)) {
			for (NodeControl node : hostNodesControls) {
				((InputNodeControl) node)
						.setNodeHighlightColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_NODEHIGHLIGHTCOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(
						InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_NODEFOLDSIGNCOLOR)) {
			for (NodeControl node : hostNodesControls) {
				((InputNodeControl) node)
						.setFoldSignColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_NODEFOLDSIGNCOLOR)));
			}
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_LINEWIDTH)) {
			this.hostLineWidth = (Integer) event.getNewValue();
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_LINECOLOR)) {
			this.hostLineColor = new Color(
					getDisplay(),
					PreferenceConverter
							.getColor(
									store,
									InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_LINECOLOR));
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_FONT)) {
			this.hostFont = new Font(
					getDisplay(),
					PreferenceConverter
							.getFontData(
									store,
									InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_FONT));
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_FONTCOLOR)) {
			this.hostFontColor = new Color(
					getDisplay(),
					PreferenceConverter
							.getColor(
									store,
									InputPreferenceConstants.INPUT_VIEWER_NAME_HOST_FONTCOLOR));
			return;
		}

		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEWIDTH)) {
			for (NodeControl node : parasiteNodesControls) {
				((InputNodeControl) node).setWidth((Integer) event
						.getNewValue());
			}
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEHEIGHT)) {
			for (NodeControl node : parasiteNodesControls) {
				((InputNodeControl) node).setHeight((Integer) event
						.getNewValue());
			}
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODECOLOR)) {
			for (NodeControl node : parasiteNodesControls) {
				((InputNodeControl) node)
						.setNodeColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODECOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(
						InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEOUTLINECOLOR)) {
			for (NodeControl node : parasiteNodesControls) {
				((InputNodeControl) node)
						.setNodeOutlineColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEOUTLINECOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(
						InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEHIGHLIGHTCOLOR)) {
			for (NodeControl node : parasiteNodesControls) {
				((InputNodeControl) node)
						.setNodeHighlightColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEHIGHLIGHTCOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(
						InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEFOLDSIGNCOLOR)) {
			for (NodeControl node : parasiteNodesControls) {
				((InputNodeControl) node)
						.setFoldSignColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEFOLDSIGNCOLOR)));
			}
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_LINEWIDTH)) {
			this.parasiteLineWidth = (Integer) event.getNewValue();
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_LINECOLOR)) {
			this.parasiteLineColor = new Color(
					getDisplay(),
					PreferenceConverter
							.getColor(
									store,
									InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_LINECOLOR));
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_FONT)) {
			this.parasiteFont = new Font(
					getDisplay(),
					PreferenceConverter
							.getFontData(
									store,
									InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_FONT));
			return;
		}
		if (event.getProperty().equals(
				InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_FONTCOLOR)) {
			this.parasiteFontColor = new Color(
					getDisplay(),
					PreferenceConverter
							.getColor(
									store,
									InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_FONTCOLOR));
			return;
		}
	}

	/**
	 * initialize the visual representation objects of the cophylogeny
	 * 
	 * @param initNodes
	 *            true if the Controls representing the nodes should be
	 *            initialized
	 * @param arrangement
	 *            the InputTreeArrangement containing information about the tree
	 *            locations or null if the node positions should not be
	 *            recalculated
	 */

	public void initCophylogeny(boolean initNodes, ITreeArrangement arrangement) {
		this.container.setRedraw(false);

		// recalculate the node positions
		if (arrangement != null) {
			this.getCophylogeny().getHostRoot().renumberNodes(0);
			this.getCophylogeny().getHostRoot().calcNodePositions(0);
			this.getCophylogeny().getParasiteRoot().renumberNodes(0);
			this.getCophylogeny().getParasiteRoot().calcNodePositions(0);
		}
		// initialize the visual controls
		if (initNodes) {
			this.associations = new ArrayList<InputHostParasiteNodeControlAssociation>();
			for (NodeControl canvas : this.hostNodesControls) {
				canvas.setMenu(null);
				canvas.dispose();
			}
			this.hostNodesControls.clear();
			initHostNodes(this.getCophylogeny().getHostRoot(), null);
			for (NodeControl canvas : this.parasiteNodesControls) {
				canvas.setMenu(null);
				canvas.dispose();
			}
			this.parasiteNodesControls.clear();
			initParasiteNodes(this.getCophylogeny().getParasiteRoot(), null);
		}
		if (arrangement != null) {
			if (arrangement.getArrangementFlag() == InputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
				if (arrangement.getPositionFlag() == InputTreeArrangement.POSITION_FIRST_HOST_SECOND_PARASITE) {
					rotateRight(InputViewerConstants.TREE_SELECTION_HOST);
					rotateLeft(InputViewerConstants.TREE_SELECTION_PARASITE);
				} else {
					rotateRight(InputViewerConstants.TREE_SELECTION_PARASITE);
					rotateLeft(InputViewerConstants.TREE_SELECTION_HOST);
				}
				if (arrangement
						.isTreeFlipped(InputViewerConstants.TREE_SELECTION_HOST)) {
					flipVertical(InputViewerConstants.TREE_SELECTION_HOST);
				}
				if (arrangement
						.isTreeFlipped(InputViewerConstants.TREE_SELECTION_PARASITE)) {
					flipVertical(InputViewerConstants.TREE_SELECTION_PARASITE);
				}
			} else {
				if (arrangement.getPositionFlag() == InputTreeArrangement.POSITION_FIRST_HOST_SECOND_PARASITE) {
					flipVertical(InputViewerConstants.TREE_SELECTION_HOST);
					flipHorizontal(InputViewerConstants.TREE_SELECTION_HOST);
				} else {
					flipVertical(InputViewerConstants.TREE_SELECTION_PARASITE);
					flipHorizontal(InputViewerConstants.TREE_SELECTION_PARASITE);
				}
				if (arrangement
						.isTreeFlipped(InputViewerConstants.TREE_SELECTION_HOST)) {
					flipHorizontal(InputViewerConstants.TREE_SELECTION_HOST);
				}
				if (arrangement
						.isTreeFlipped(InputViewerConstants.TREE_SELECTION_PARASITE)) {
					flipHorizontal(InputViewerConstants.TREE_SELECTION_PARASITE);
				}
			}
			GC gc = new GC(this.container);
			// move the host tree to its original position
			int difX = arrangement
					.getTreeRootPosition(InputViewerConstants.TREE_SELECTION_HOST).x
					- this.hostNodesControls.get(0).getNode().getPosX();
			int difY = arrangement
					.getTreeRootPosition(InputViewerConstants.TREE_SELECTION_HOST).y
					- this.hostNodesControls.get(0).getNode().getPosY();
			((InputNodeControl) this.hostNodesControls.get(0)).movePositionDif(
					scale(difX), scale(difY), gc, true);
			// move the host tree down and to the right so that each node has a
			// position grater than (0,0)
			Rectangle hostRect = getScaledTreeSize(InputViewerConstants.TREE_SELECTION_HOST);
			((InputNodeControl) this.hostNodesControls.get(0)).movePositionDif(
					Math.max(0, -hostRect.x), Math.max(0, -hostRect.y), gc,
					true);

			// move the parasite tree to its original position
			difX = arrangement
					.getTreeRootPosition(InputViewerConstants.TREE_SELECTION_PARASITE).x
					- this.parasiteNodesControls.get(0).getNode().getPosX();
			difY = arrangement
					.getTreeRootPosition(InputViewerConstants.TREE_SELECTION_PARASITE).y
					- this.parasiteNodesControls.get(0).getNode().getPosY();
			((InputNodeControl) this.parasiteNodesControls.get(0))
					.movePositionDif(scale(difX), scale(difY), gc, true);
			// move the parasite tree down and to the right so that each node
			// has a position grater than (0,0)
			Rectangle parasiteRect = getScaledTreeSize(InputViewerConstants.TREE_SELECTION_PARASITE);
			((InputNodeControl) this.parasiteNodesControls.get(0))
					.movePositionDif(Math.max(0, -parasiteRect.x), Math.max(0,
							-parasiteRect.y), gc, true);

			// move the second of the tree down or to the right, so that they
			// have distance to each other of at least
			// InputViewerConstants.TREE_DISTANCE
			IPreferenceStore store = PlatformUI.getPreferenceStore();
			hostRect = getNormalizedTreeSize(InputViewerConstants.TREE_SELECTION_HOST);
			parasiteRect = getNormalizedTreeSize(InputViewerConstants.TREE_SELECTION_PARASITE);
			int treeDistance = store
					.getInt(InputPreferenceConstants.INPUT_VIEWER_NAME_REARRANGE_TREE_DISTANCE);
			if (arrangement.getArrangementFlag() == InputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
				if (arrangement.getPositionFlag() == InputTreeArrangement.POSITION_FIRST_HOST_SECOND_PARASITE) {
					if (parasiteRect.x - treeDistance < hostRect.width) {
						((InputNodeControl) this.parasiteNodesControls.get(0))
								.movePositionDif(treeDistance
										- (parasiteRect.x - hostRect.width), 0,
										gc, true);
					}
				} else {
					if (hostRect.x - treeDistance < parasiteRect.width) {
						((InputNodeControl) this.hostNodesControls.get(0))
								.movePositionDif(treeDistance
										- (hostRect.x - parasiteRect.width), 0,
										gc, true);
					}
				}
			} else {
				if (arrangement.getPositionFlag() == InputTreeArrangement.POSITION_FIRST_HOST_SECOND_PARASITE) {
					if (parasiteRect.y - treeDistance < hostRect.height) {
						((InputNodeControl) this.parasiteNodesControls.get(0))
								.movePositionDif(0, treeDistance
										- (parasiteRect.y - hostRect.height),
										gc, true);
					}
				} else {
					if (hostRect.y - treeDistance < parasiteRect.height) {
						((InputNodeControl) this.hostNodesControls.get(0))
								.movePositionDif(0, treeDistance
										- (hostRect.y - parasiteRect.height),
										gc, true);
					}
				}
			}
		} else if (!getCophylogeny().isPositionsApplied()) {
			// set scale factor to 1.0
			this.scaleFactor = 1d;
			this.getCophylogeny().getHostRoot().calcNodePositions(0);
			this.getCophylogeny().getParasiteRoot().calcNodePositions(0);
			for (NodeControl node : this.hostNodesControls) {
				node.setScaleFactor(scaleFactor);
			}
			for (NodeControl node : this.parasiteNodesControls) {
				node.setScaleFactor(scaleFactor);
			}
			GC gc = new GC(this.container);
			// rotate host tree so that the root is in the left side
			rotateRight(InputViewerConstants.TREE_SELECTION_HOST);
			Rectangle rectHost = getScaledTreeSize(InputViewerConstants.TREE_SELECTION_HOST);
			// rotate the parasite tree so that the root is on the right side
			rotateLeft(InputViewerConstants.TREE_SELECTION_PARASITE);
			flipVertical(InputViewerConstants.TREE_SELECTION_PARASITE);
			// move the parasite tree to the right so that there are 100 pixel
			// between the two trees
			((InputNodeControl) this.parasiteNodesControls.get(0))
					.movePositionDif(rectHost.width + 100, 0, gc, false);
			Rectangle rectParasite = getScaledTreeSize(InputViewerConstants.TREE_SELECTION_PARASITE);
			// move the smaller of both trees down so that this one is centered
			// comparing to the other one
			if (rectHost.height < rectParasite.height) {
				((InputNodeControl) this.hostNodesControls.get(0))
						.movePositionDif(0,
								(rectParasite.height - rectHost.height) / 2,
								gc, false);
			} else {
				((InputNodeControl) this.parasiteNodesControls.get(0))
						.movePositionDif(0,
								(rectHost.height - rectParasite.height) / 2,
								gc, false);
			}
			getCophylogeny().setPositionsApplied(true);
		}
		if (arrangement == null) {
			Rectangle rect = getScaledTreeSize(InputViewerConstants.TREE_SELECTION_BOTH);
			while (this.scaleFactor > 0.01
					&& (rect.height > 768 || rect.width > 1024)) {
				zoomOut();
				rect = getScaledTreeSize(InputViewerConstants.TREE_SELECTION_BOTH);
			}
		}
		resetMinSize();
		this.container.setRedraw(true);
	}

	public ITreeArrangement getTreeArrangement() {
		ITreeArrangement arrangement = new InputTreeArrangement(this
				.getCophylogeny());
		return arrangement;
	}

	/**
	 * initialize the nodes of the host tree
	 * 
	 * @param hostNode
	 * @param parent
	 * @return
	 */
	private InputHostNodeControl initHostNodes(IPositionHostNode hostNode,
			InputNodeControl parent) {
		InputHostNodeControl nodeControl = new InputHostNodeControl(this,
				hostNode, parent);
		int posX = scale(hostNode.getPosX());
		int posY = scale(hostNode.getPosY());
		int width = scale(nodeControl.width);
		int height = scale(nodeControl.height);
		nodeControl.setScaleFactor(this.scaleFactor);
		nodeControl.setBounds(posX, posY, width, height);
		nodeControl.setParentNodeControl(parent);
		if (getNodeContextMenu() != null)
			nodeControl.setMenu(getNodeContextMenu());
		this.hostNodesControls.add(nodeControl);

		for (IPositionHostNode childNode : hostNode.getChildren()) {
			nodeControl.getChildNodeControls().add(
					initHostNodes(childNode, nodeControl));
		}
		return nodeControl;
	}

	/**
	 * initialize the nodes of the host tree
	 * 
	 * @param hostNode
	 * @param parent
	 * @return
	 */
	private InputParasiteNodeControl initParasiteNodes(
			IPositionParasiteNode parasiteNode, InputNodeControl parent) {
		InputParasiteNodeControl nodeControl = new InputParasiteNodeControl(
				this, parasiteNode, parent);
		int posX = scale(parasiteNode.getPosX());
		int posY = scale(parasiteNode.getPosY());
		int width = scale(nodeControl.width);
		int height = scale(nodeControl.height);
		nodeControl.setScaleFactor(this.scaleFactor);
		nodeControl.setBounds(posX, posY, width, height);
		nodeControl.setParentNodeControl(parent);
		if (getNodeContextMenu() != null)
			nodeControl.setMenu(getNodeContextMenu());
		List<IHostNode> mappedNodesToDelete = new ArrayList<IHostNode>();
		for (IHostNode mappedNode : parasiteNode.getAssociations()) {
			InputNodeControl hostControl = (InputNodeControl) getNodeControlByINode(mappedNode);
			if (hostControl == null) {
				mappedNodesToDelete.add(mappedNode);
			} else {
				InputHostParasiteNodeControlAssociation association = new InputHostParasiteNodeControlAssociation(
						(InputHostNodeControl) getNodeControlByINode(mappedNode),
						nodeControl);
				this.associations.add(association);
			}
		}
		for (IHostNode mappedNode : mappedNodesToDelete) {
			parasiteNode.getAssociations().remove(mappedNode);
		}
		this.parasiteNodesControls.add(nodeControl);

		for (IPositionParasiteNode childNode : parasiteNode.getChildren()) {
			nodeControl.getChildNodeControls().add(
					initParasiteNodes(childNode, nodeControl));
		}
		return nodeControl;
	}

	/**
	 * draw the lines of the specified tree
	 * 
	 * @param gc
	 */
	private void drawNodeLines(int tree, GC gc) {
		gc.setAntialias(SWT.ON);
		List<NodeControl> nodes = new ArrayList<NodeControl>();
		int arrangement = InputTreeArrangement.ARRANGEMENT_HORIZONTAL;
		gc.setLineCap(SWT.CAP_ROUND);
		gc.setLineJoin(SWT.JOIN_ROUND);
		switch (tree) {
		case InputViewerConstants.TREE_SELECTION_HOST:
			gc.setLineWidth(scale(hostLineWidth));
			gc.setForeground(hostLineColor);
			nodes = this.hostNodesControls;
			arrangement = InputTreeArrangement.getArrangement(this
					.getCophylogeny().getHostRoot());
			break;
		case InputViewerConstants.TREE_SELECTION_PARASITE:
			gc.setLineWidth(scale(parasiteLineWidth));
			gc.setForeground(parasiteLineColor);
			nodes = this.parasiteNodesControls;
			arrangement = InputTreeArrangement.getArrangement(this
					.getCophylogeny().getParasiteRoot());
			break;
		default:
			break;
		}

		if (linetype
				.equals(InputPreferenceConstants.INPUT_VIEWER_LINETYPE_ACUTE_ANGLED)) {
			// diagonal lines
			for (NodeControl node : nodes) {
				NodeControl parent = node.getParentNodeControl();
				if (parent != null && node.isVisible()) {
					gc.drawLine(scale(parent.getNode().getPosX() + parent.width
							/ 2), scale(parent.getNode().getPosY()
							+ parent.height / 2), scale(node.getNode()
							.getPosX()
							+ node.width / 2), scale(node.getNode().getPosY()
							+ node.height / 2));
				}
			}
		} else {
			// orthogonal lines
			if (arrangement == InputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null && node.isVisible()) {
						gc.drawLine(scale(parent.getNode().getPosX()
								+ parent.width / 2), scale(parent.getNode()
								.getPosY()
								+ parent.height / 2), scale(parent.getNode()
								.getPosX()
								+ parent.width / 2), scale(node.getNode()
								.getPosY()
								+ node.height / 2));
						gc.drawLine(scale(parent.getNode().getPosX()
								+ parent.width / 2), scale(node.getNode()
								.getPosY()
								+ node.height / 2), scale(node.getNode()
								.getPosX()
								+ node.width / 2), scale(node.getNode()
								.getPosY()
								+ node.height / 2));
					}
				}
			} else if (arrangement == InputTreeArrangement.ARRANGEMENT_VERTICAL) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null && node.isVisible()) {
						gc.drawLine(scale(parent.getNode().getPosX()
								+ parent.width / 2), scale(parent.getNode()
								.getPosY()
								+ parent.height / 2), scale(node.getNode()
								.getPosX()
								+ node.width / 2), scale(parent.getNode()
								.getPosY()
								+ parent.height / 2));
						gc.drawLine(scale(node.getNode().getPosX() + node.width
								/ 2), scale(parent.getNode().getPosY()
								+ parent.height / 2), scale(node.getNode()
								.getPosX()
								+ node.width / 2), scale(node.getNode()
								.getPosY()
								+ node.height / 2));
					}
				}
			}
		}
	}

	/**
	 * draw the lines of the specified tree
	 * 
	 * @param gc
	 */
	private void drawNodeLines(int tree, Graphics2D g2d) {
		List<NodeControl> nodes = new ArrayList<NodeControl>();
		int arrangement = InputTreeArrangement.ARRANGEMENT_HORIZONTAL;
		RGB rgb;
		switch (tree) {
		case InputViewerConstants.TREE_SELECTION_HOST:
			g2d.setStroke(new BasicStroke(hostLineWidth));
			rgb = hostLineColor.getRGB();
			g2d.setColor(new java.awt.Color(rgb.red, rgb.green, rgb.blue));
			nodes = this.hostNodesControls;
			arrangement = InputTreeArrangement.getArrangement(this
					.getCophylogeny().getHostRoot());
			break;
		case InputViewerConstants.TREE_SELECTION_PARASITE:
			g2d.setStroke(new BasicStroke(parasiteLineWidth));
			rgb = parasiteLineColor.getRGB();
			g2d.setColor(new java.awt.Color(rgb.red, rgb.green, rgb.blue));
			nodes = this.parasiteNodesControls;
			arrangement = InputTreeArrangement.getArrangement(this
					.getCophylogeny().getParasiteRoot());
			break;
		default:
			break;
		}

		if (linetype
				.equals(InputPreferenceConstants.INPUT_VIEWER_LINETYPE_ACUTE_ANGLED)) {
			// diagonal lines
			for (NodeControl node : nodes) {
				NodeControl parent = node.getParentNodeControl();
				if (parent != null && node.isVisible()) {
					g2d.drawLine(parent.getNode().getPosX() + parent.width / 2,
							parent.getNode().getPosY() + parent.height / 2,
							node.getNode().getPosX() + node.width / 2, node
									.getNode().getPosY()
									+ node.height / 2);
				}
			}
		}

		else {
			// orthogonal lines
			if (arrangement == InputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null) {
						g2d
								.drawLine(parent.getNode().getPosX()
										+ parent.width / 2, parent.getNode()
										.getPosY()
										+ parent.height / 2, parent.getNode()
										.getPosX()
										+ parent.width / 2, node.getNode()
										.getPosY()
										+ node.height / 2);
						g2d.drawLine(parent.getNode().getPosX() + parent.width
								/ 2,
								node.getNode().getPosY() + node.height / 2,
								node.getNode().getPosX() + node.width / 2, node
										.getNode().getPosY()
										+ node.height / 2);
					}
				}
			} else if (arrangement == InputTreeArrangement.ARRANGEMENT_VERTICAL) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null) {
						g2d.drawLine(parent.getNode().getPosX() + parent.width
								/ 2, parent.getNode().getPosY() + parent.height
								/ 2, node.getNode().getPosX() + node.width / 2,
								parent.getNode().getPosY() + parent.height / 2);
						g2d.drawLine(node.getNode().getPosX() + node.width / 2,
								parent.getNode().getPosY() + parent.height / 2,
								node.getNode().getPosX() + node.width / 2, node
										.getNode().getPosY()
										+ node.height / 2);
					}
				}
			}
		}
	}

	/**
	 * draw the lines of the association between host and parasite tree
	 * 
	 * @param gc
	 */
	private void drawAssociationLines(GC gc) {
		gc.setAntialias(SWT.ON);
		gc.setLineCap(SWT.CAP_SQUARE);
		gc.setLineJoin(SWT.JOIN_ROUND);
		int[] dashes = { Math.max(2, scale(10)), Math.max(1, scale(5)) };
		gc.setLineDash(dashes);
		gc.setLineWidth(scale(associationLineWidth));
		gc.setForeground(associationLineColor);
		for (InputHostParasiteNodeControlAssociation association : this.associations) {
			if (association.getHost().isVisible()
					&& association.getParasite().isVisible()) {
				int startX = scale(association.getHost().getPosX()
						+ association.getHost().width / 2);
				int startY = scale(association.getHost().getPosY()
						+ association.getHost().height / 2);
				int endX = scale(association.getParasite().getPosX()
						+ association.getParasite().width / 2);
				int endY = scale(association.getParasite().getPosY()
						+ association.getParasite().height / 2);
				gc.drawLine(startX, startY, endX, endY);
			}
		}

		if (this.phiSource != null) {
			int startX = scale(phiSource.getPosX() + phiSource.width / 2);
			int startY = scale(phiSource.getPosY() + phiSource.height / 2);
			int endX = this.phiTargetPosition.x;
			int endY = this.phiTargetPosition.y;
			if (this.phiAssociationExists)
				gc.setForeground(new Color(Display.getDefault(), 255, 0, 0));
			gc.drawLine(startX, startY, endX, endY);
			this.phiAssociationExists = false;
		}
		int[] dashes2 = { 1 };
		gc.setLineDash(dashes2);
	}

	/**
	 * draw the lines of the association between host and parasite tree
	 * 
	 * @param gc
	 */
	private void drawAssociationLines(Graphics2D g2d) {
		float[] dashes = { 10f, 5f };
		g2d.setStroke(new BasicStroke(associationLineWidth,
				BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 1f, dashes, 0f));
		RGB rgb = associationLineColor.getRGB();
		g2d.setColor(new java.awt.Color(rgb.red, rgb.green, rgb.blue));
		for (InputHostParasiteNodeControlAssociation association : this.associations) {
			int startX = association.getHost().getPosX()
					+ association.getHost().width / 2;
			int startY = association.getHost().getPosY()
					+ association.getHost().height / 2;
			int endX = association.getParasite().getPosX()
					+ association.getParasite().width / 2;
			int endY = association.getParasite().getPosY()
					+ association.getParasite().height / 2;
			g2d.drawLine(startX, startY, endX, endY);
		}
		g2d.setStroke(new BasicStroke(associationLineWidth));
	}

	/**
	 * draw the labels for the nodes of the given tree
	 * 
	 * @param tree
	 * @param gc
	 */
	private void drawLabels(int tree, GC gc) {
		gc.setAntialias(SWT.ON);
		List<NodeControl> nodes = new ArrayList<NodeControl>();
		int arrangement = InputTreeArrangement.ARRANGEMENT_HORIZONTAL;
		switch (tree) {
		case InputViewerConstants.TREE_SELECTION_HOST:
			gc.setForeground(hostFontColor);
			FontData hfd = hostFont.getFontData()[0];
			Font hf = new Font(getDisplay(), hfd.getName(), Math.max(1,
					scale(hfd.getHeight())), hfd.getStyle());
			gc.setFont(hf);
			nodes = this.hostNodesControls;
			arrangement = InputTreeArrangement.getArrangement(this
					.getCophylogeny().getHostRoot());
			break;
		case InputViewerConstants.TREE_SELECTION_PARASITE:
			gc.setForeground(parasiteFontColor);
			FontData pfd = parasiteFont.getFontData()[0];
			Font pf = new Font(getDisplay(), pfd.getName(), Math.max(1,
					scale(pfd.getHeight())), pfd.getStyle());
			gc.setFont(pf);
			nodes = this.parasiteNodesControls;
			arrangement = InputTreeArrangement.getArrangement(this
					.getCophylogeny().getParasiteRoot());
			break;
		default:
			break;
		}

		if (arrangement == InputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
			for (NodeControl node : nodes) {
				String label = node.getNode().getLabel();
				if (isShowRankLabels())
					label += " (" + node.getNode().getRankText() + ")";
				if (node.isVisible()) {
					gc.drawText(label, scale(node.getNode().getPosX()
							+ node.width), scale(node.getNode().getPosY() - 6),
							true);
				}
			}
		} else if (arrangement == InputTreeArrangement.ARRANGEMENT_VERTICAL) {
			Transform t = new Transform(gc.getDevice());
			t.rotate(90f);
			gc.setTransform(t);
			for (NodeControl node : nodes) {
				String label = node.getNode().getLabel();
				if (isShowRankLabels())
					label += " (" + node.getNode().getRankText() + ")";
				if (node.isVisible()) {
					gc.drawText(label, scale(node.getNode().getPosY()
							+ node.width),
							-scale(node.getNode().getPosX() + 24), true);
				}
			}
			t.rotate(-90f);
			gc.setTransform(t);
			t.dispose();
		}
	}

	/**
	 * draw the labels for the nodes of the given tree
	 * 
	 * @param tree
	 * @param gc
	 */
	private void drawLabels(int tree, Graphics2D g2d) {
		List<NodeControl> nodes = new ArrayList<NodeControl>();
		int arrangement = InputTreeArrangement.ARRANGEMENT_HORIZONTAL;
		RGB rgb;
		FontData fontData;
		switch (tree) {
		case InputViewerConstants.TREE_SELECTION_HOST:
			rgb = hostFontColor.getRGB();
			g2d.setColor(new java.awt.Color(rgb.red, rgb.green, rgb.blue));
			fontData = hostFont.getFontData()[0];
			g2d.setFont(new java.awt.Font(fontData.getName(), fontData
					.getStyle(), fontData.getHeight()));
			nodes = this.hostNodesControls;
			arrangement = InputTreeArrangement.getArrangement(this
					.getCophylogeny().getHostRoot());
			break;
		case InputViewerConstants.TREE_SELECTION_PARASITE:
			rgb = parasiteFontColor.getRGB();
			g2d.setColor(new java.awt.Color(rgb.red, rgb.green, rgb.blue));
			fontData = parasiteFont.getFontData()[0];
			g2d.setFont(new java.awt.Font(fontData.getName(), fontData
					.getStyle(), fontData.getHeight()));
			nodes = this.parasiteNodesControls;
			arrangement = InputTreeArrangement.getArrangement(this
					.getCophylogeny().getParasiteRoot());
			break;
		default:
			break;
		}

		if (arrangement == InputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
			for (NodeControl node : nodes) {
				if (node.getNode().isLeaf()) {
					String label = node.getNode().getLabel();
					if (this.getCophylogeny().getOptions().get(
							OptionsConstants.RANK_ID) != null
							&& this.getCophylogeny().getOptions().get(
									OptionsConstants.RANK_ID).equals(
									OptionsConstants.RANK_1_ID))
						label += " (" + node.getNode().getRankText() + ")";
					g2d.drawString(label,
							node.getNode().getPosX() + node.width, node
									.getNode().getPosY() + 10);
				}
			}
		} else if (arrangement == InputTreeArrangement.ARRANGEMENT_VERTICAL) {
			AffineTransform t = new AffineTransform();
			t.rotate(90d * Math.PI / 180);
			g2d.setTransform(t);
			for (NodeControl node : nodes) {
				if (node.getNode().isLeaf()) {
					String label = node.getNode().getLabel();
					if (this.getCophylogeny().getOptions().get(
							OptionsConstants.RANK_ID) != null
							&& this.getCophylogeny().getOptions().get(
									OptionsConstants.RANK_ID).equals(
									OptionsConstants.RANK_1_ID))
						label += " (" + node.getNode().getRankText() + ")";
					g2d.drawString(label,
							node.getNode().getPosY() + node.width, -node
									.getNode().getPosX() - 10);
				}
			}
			t.rotate(-90d * Math.PI / 180);
			g2d.setTransform(t);
		}
	}

	/**
	 * flips the tree horizontal
	 * 
	 * @param tree
	 */
	private void flipHorizontal(int tree) {
		this.container.setRedraw(false);
		Rectangle rect = getNormalizedTreeSize(tree);
		switch (tree) {
		case InputViewerConstants.TREE_SELECTION_HOST:
			for (NodeControl node : this.hostNodesControls) {
				node.flipHorizontal(rect);
			}
			break;
		case InputViewerConstants.TREE_SELECTION_PARASITE:
			for (NodeControl node : this.parasiteNodesControls) {
				node.flipHorizontal(rect);
			}
			break;
		case InputViewerConstants.TREE_SELECTION_BOTH:
			for (NodeControl node : this.hostNodesControls) {
				node.flipHorizontal(rect);
			}
			for (NodeControl node : this.parasiteNodesControls) {
				node.flipHorizontal(rect);
			}
			break;
		}
		this.resetMinSize();
		this.container.setRedraw(true);
	}

	/**
	 * flips the tree vertical
	 * 
	 * @param tree
	 */
	private void flipVertical(int tree) {
		this.container.setRedraw(false);
		Rectangle rect = getNormalizedTreeSize(tree);
		switch (tree) {
		case InputViewerConstants.TREE_SELECTION_HOST:
			for (NodeControl node : this.hostNodesControls) {
				node.flipVertical(rect);
			}
			break;
		case InputViewerConstants.TREE_SELECTION_PARASITE:
			for (NodeControl node : this.parasiteNodesControls) {
				node.flipVertical(rect);
			}
			break;
		case InputViewerConstants.TREE_SELECTION_BOTH:
			for (NodeControl node : this.hostNodesControls) {
				node.flipVertical(rect);
			}
			for (NodeControl node : this.parasiteNodesControls) {
				node.flipVertical(rect);
			}
			break;
		}
		this.resetMinSize();
		this.container.setRedraw(true);
	}

	/**
	 * rotate the tree 90 degree to the left
	 * 
	 * @param tree
	 */
	private void rotateLeft(int tree) {
		this.container.setRedraw(false);
		Rectangle rect = getNormalizedTreeSize(tree);
		switch (tree) {
		case InputViewerConstants.TREE_SELECTION_HOST:
			for (NodeControl node : this.hostNodesControls) {
				node.rotate(rect);
			}
			flipVertical(tree);
			break;
		case InputViewerConstants.TREE_SELECTION_PARASITE:
			for (NodeControl node : this.parasiteNodesControls) {
				node.rotate(rect);
			}
			flipVertical(tree);
			break;
		case InputViewerConstants.TREE_SELECTION_BOTH:
			for (NodeControl node : this.hostNodesControls) {
				node.rotate(rect);
			}
			for (NodeControl node : this.parasiteNodesControls) {
				node.rotate(rect);
			}
			flipVertical(tree);
			break;
		}
		this.resetMinSize();
		this.container.setRedraw(true);
	}

	/**
	 * rotate the tree 90 degree to the right
	 * 
	 * @param tree
	 */
	private void rotateRight(int tree) {
		this.container.setRedraw(false);
		Rectangle rect = getNormalizedTreeSize(tree);
		switch (tree) {
		case InputViewerConstants.TREE_SELECTION_HOST:
			for (NodeControl node : this.hostNodesControls) {
				node.rotate(rect);
			}
			flipHorizontal(InputViewerConstants.TREE_SELECTION_HOST);
			break;
		case InputViewerConstants.TREE_SELECTION_PARASITE:
			for (NodeControl node : this.parasiteNodesControls) {
				node.rotate(rect);
			}
			flipHorizontal(InputViewerConstants.TREE_SELECTION_PARASITE);
			break;
		case InputViewerConstants.TREE_SELECTION_BOTH:
			for (NodeControl node : this.hostNodesControls) {
				node.rotate(rect);
			}
			for (NodeControl node : this.parasiteNodesControls) {
				node.rotate(rect);
			}
			flipHorizontal(tree);
			break;
		}
		this.resetMinSize();
		this.container.setRedraw(true);
	}

	/**
	 * @return the treeMode
	 */
	public int getTreeMode() {
		return treeMode;
	}

	/**
	 * @param treeMode
	 *            the treeMode to set
	 */
	public void setTreeMode(int treeMode) {
		int oldValue = this.treeMode;
		this.treeMode = treeMode;
		firePropertyChange(InputViewerConstants.PROPERTY_TREE_MODE, oldValue,
				treeMode);
	}

	/**
	 * @return the editMode
	 */
	public int getEditMode() {
		return editMode;
	}

	/**
	 * @param editMode
	 *            the editMode to set
	 */
	public void setEditMode(int editMode) {
		int oldValue = this.editMode;
		this.editMode = editMode;
		firePropertyChange(InputViewerConstants.PROPERTY_EDIT_MODE, oldValue,
				editMode);
	}

	/**
	 * Flips the selected tree horizontally.
	 */
	public void flipHorizontal() {
		flipHorizontal(this.treeMode);
		setDirty(true);
	}

	/**
	 * Flips the selected tree vertically.
	 */
	public void flipVertical() {
		flipVertical(this.treeMode);
		setDirty(true);
	}

	/**
	 * Rotate the selected tree to the left.
	 */
	public void rotateLeft() {
		rotateLeft(this.treeMode);
		setDirty(true);
	}

	/**
	 * Rotate the selected tree to the right.
	 */
	public void rotateRight() {
		rotateRight(this.treeMode);
		setDirty(true);
	}

	/**
	 * @return the cophylogeny
	 */
	public InputCophylogeny getCophylogeny() {
		return (InputCophylogeny) cophylogeny;
	}

	/**
	 * @param cophylogeny
	 *            the cophylogeny to set
	 */
	public void setCophylogeny(InputCophylogeny cophylogeny) {
		this.cophylogeny = cophylogeny;
	}

	/**
	 * @return the phiStart
	 */
	public InputNodeControl getPhiSource() {
		return phiSource;
	}

	/**
	 * @param phiSource
	 *            the phiStart to set
	 */
	public void setPhiSource(InputNodeControl phiSource) {
		this.phiSource = phiSource;
		if (phiSource != null)
			setPhiTargetPosition(new Point(scale(phiSource.getPosX()
					+ phiSource.width / 2), scale(phiSource.getPosY()
					+ phiSource.height / 2)));
	}

	/**
	 * @return the mousePosition
	 */
	public Point getPhiTargetPosition() {
		return phiTargetPosition;
	}

	/**
	 * @param phiTargetPosition
	 *            the mousePosition to set
	 */
	public void setPhiTargetPosition(Point phiTargetPosition) {
		this.phiTargetPosition = phiTargetPosition;
	}

	/**
	 * @return the phiAssociationExists
	 */
	public boolean isPhiAssociationExists() {
		return phiAssociationExists;
	}

	/**
	 * @param phiAssociationExists
	 *            the phiAssociationExists to set
	 */
	public void setPhiAssociationExists(boolean phiAssociationExists) {
		this.phiAssociationExists = phiAssociationExists;
	}

	/**
	 * @return the associations
	 */
	public List<InputHostParasiteNodeControlAssociation> getAssociations() {
		return associations;
	}

	/**
	 * @param host
	 * @return the associations containing this host
	 */
	public List<InputHostParasiteNodeControlAssociation> getAssociations(
			InputHostNodeControl host) {
		List<InputHostParasiteNodeControlAssociation> matchingAssociations = new ArrayList<InputHostParasiteNodeControlAssociation>();
		for (InputHostParasiteNodeControlAssociation association : this.associations) {
			if (association.getHost().equals(host))
				matchingAssociations.add(association);
		}
		return matchingAssociations;
	}

	/**
	 * @param host
	 * @return the associations containing this parasite
	 */
	public List<InputHostParasiteNodeControlAssociation> getAssociations(
			InputParasiteNodeControl parasite) {
		List<InputHostParasiteNodeControlAssociation> matchingAssociations = new ArrayList<InputHostParasiteNodeControlAssociation>();
		for (InputHostParasiteNodeControlAssociation association : this.associations) {
			if (association.getParasite().equals(parasite))
				matchingAssociations.add(association);
		}
		return matchingAssociations;
	}

	/**
	 * 
	 */
	public void updateAssociations() {
		this.associations.clear();
		for (NodeControl parasiteControl : this.parasiteNodesControls) {
			for (IHostNode mappedNode : ((InputParasiteNodeControl) parasiteControl)
					.getNode().getAssociations()) {
				NodeControl hostControl = getNodeControlByINode(mappedNode);
				InputHostParasiteNodeControlAssociation association = new InputHostParasiteNodeControlAssociation(
						(InputHostNodeControl) hostControl,
						(InputParasiteNodeControl) parasiteControl);
				this.associations.add(association);
			}
		}
	}

	/**
	 * @param associations
	 *            the associations to set
	 */
	public void setAssociations(
			List<InputHostParasiteNodeControlAssociation> associations) {
		this.associations = associations;
	}

	public void paintContent(Graphics2D g2d) {
		drawNodeLines(InputViewerConstants.TREE_SELECTION_HOST, g2d);
		drawNodeLines(InputViewerConstants.TREE_SELECTION_PARASITE, g2d);
		drawAssociationLines(g2d);
		super.paintContent(g2d);
		drawLabels(InputViewerConstants.TREE_SELECTION_HOST, g2d);
		drawLabels(InputViewerConstants.TREE_SELECTION_PARASITE, g2d);
	}

}
