/**
 * 
 */
package com.wieseke.cptk.input.viewer;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.common.api.IPositionNode;
import com.wieseke.cptk.common.viewer.CophylogenyViewer;
import com.wieseke.cptk.common.viewer.NodeControl;
import com.wieseke.cptk.input.dao.InputParasiteNode;
import com.wieseke.cptk.input.operation.EditAddParasiteAssociationOperation;
import com.wieseke.cptk.input.preferences.InputPreferenceConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class InputParasiteNodeControl extends InputNodeControl {

	/**
	 * default constructor
	 * 
	 * @param viewer
	 * @param node
	 * @param parent
	 */
	public InputParasiteNodeControl(CophylogenyViewer viewer,
			IPositionNode node, NodeControl parent) {
		super(viewer, node, parent);
	}

	
	protected void init() {
		IPreferenceStore store = PlatformUI.getPreferenceStore();
		this.width = store
				.getInt(InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEWIDTH);
		this.height = store
				.getInt(InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEHEIGHT);
		this.foldSignColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEFOLDSIGNCOLOR));
		this.nodeColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODECOLOR));
		this.nodeHighlightColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEHIGHLIGHTCOLOR));
		this.nodeOutlineColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								InputPreferenceConstants.INPUT_VIEWER_NAME_PARASITE_NODEOUTLINECOLOR));
	}

	
	public void addAssociationEnd() {
		EditAddParasiteAssociationOperation operation = new EditAddParasiteAssociationOperation(
				this.getViewer(), this.getNode());
		operation.addContext(this.viewer.getUndoContext());
		try {
			PlatformUI.getWorkbench().getOperationSupport()
					.getOperationHistory().execute(operation, null, null);
		} catch (ExecutionException e) {
			MessageDialog
					.openError(this.viewer.getShell(), "Add Association Error",
							"Exception while adding a association to node "
									+ this.getNode().getLabel() + ": "
									+ e.getMessage());
		}
	}

	
	public InputParasiteNode getNode() {
		return (InputParasiteNode) super.getNode();
	}
}
