/**
 * 
 */
package com.wieseke.cptk.input.viewer;

import org.eclipse.swt.graphics.Point;

import com.wieseke.cptk.common.api.IPositionNode;
import com.wieseke.cptk.common.api.ITreeArrangement;
import com.wieseke.cptk.input.constants.InputViewerConstants;
import com.wieseke.cptk.input.dao.InputCophylogeny;

/**
 * @author Nicolas Wieseke
 * 
 */
public class InputTreeArrangement implements ITreeArrangement {

	public static final int ARRANGEMENT_HORIZONTAL = 0;
	public static final int ARRANGEMENT_VERTICAL = 1;

	public static final int POSITION_FIRST_HOST_SECOND_PARASITE = 0;
	public static final int POSITION_FIRST_PARASITE_SECOND_HOST = 1;

	private Point hostRootPosition;
	private Point hostFirstLeafPosition;
	private Point hostLastLeafPosition;
	private Point parasiteRootPosition;
	private Point parasiteFirstLeafPosition;
	private Point parasiteLastLeafPosition;

	private int arrangementFlag;
	private int positionFlag;
	private boolean hostFlipped;
	private boolean parasiteFlipped;

	public static int getArrangement(IPositionNode root) {
		IPositionNode firstLeaf = root;
		while (firstLeaf.getChildren().size() != 0) {
			firstLeaf = firstLeaf.getChildren().get(0);
		}
		Point firstLeafPosition = new Point(firstLeaf.getPosX(), firstLeaf.getPosY());
		IPositionNode lastLeaf = root;
		while (lastLeaf.getChildren().size() != 0) {
			lastLeaf = lastLeaf.getChildren().get(lastLeaf.getChildren().size() - 1);
		}
		Point lastLeafPosition = new Point(lastLeaf.getPosX(), lastLeaf.getPosY());

		int horizontalDistance = Math.abs(firstLeafPosition.x - lastLeafPosition.x);
		int verticalDistance = Math.abs(firstLeafPosition.y - lastLeafPosition.y);

		if (verticalDistance >= horizontalDistance) {
			return ARRANGEMENT_HORIZONTAL;
		} else {
			return ARRANGEMENT_VERTICAL;
		}

	}

	public InputTreeArrangement(InputCophylogeny cophylogen) {
		IPositionNode hostRoot = cophylogen.getHostRoot();
		this.hostRootPosition = new Point(hostRoot.getPosX(), hostRoot.getPosY());
		IPositionNode firstHostLeaf = hostRoot;
		while (firstHostLeaf.getChildren().size() != 0) {
			firstHostLeaf = firstHostLeaf.getChildren().get(0);
		}
		this.hostFirstLeafPosition = new Point(firstHostLeaf.getPosX(), firstHostLeaf.getPosY());
		IPositionNode lastHostLeaf = hostRoot;
		while (lastHostLeaf.getChildren().size() != 0) {
			lastHostLeaf = lastHostLeaf.getChildren().get(lastHostLeaf.getChildren().size() - 1);
		}
		this.hostLastLeafPosition = new Point(lastHostLeaf.getPosX(), lastHostLeaf.getPosY());

		IPositionNode parasiteRoot = cophylogen.getParasiteRoot();
		this.parasiteRootPosition = new Point(parasiteRoot.getPosX(), parasiteRoot.getPosY());
		IPositionNode firstParasiteLeaf = parasiteRoot;
		while (firstParasiteLeaf.getChildren().size() != 0) {
			firstParasiteLeaf = firstParasiteLeaf.getChildren().get(0);
		}
		this.parasiteFirstLeafPosition = new Point(firstParasiteLeaf.getPosX(), firstParasiteLeaf.getPosY());
		IPositionNode lastParasiteLeaf = parasiteRoot;
		while (lastParasiteLeaf.getChildren().size() != 0) {
			lastParasiteLeaf = lastParasiteLeaf.getChildren().get(lastParasiteLeaf.getChildren().size() - 1);
		}
		this.parasiteLastLeafPosition = new Point(lastParasiteLeaf.getPosX(), lastParasiteLeaf.getPosY());

		// int rootXDistance = Math.abs(this.hostRootPosition.x
		// - this.parasiteRootPosition.x);
		// int rootYDistance = Math.abs(this.hostRootPosition.y
		// - this.parasiteRootPosition.y);

		int horizontalDistance = Math.abs(this.hostFirstLeafPosition.x - this.hostLastLeafPosition.x)
				+ Math.abs(this.parasiteFirstLeafPosition.x - this.parasiteLastLeafPosition.x);
		int verticalDistance = Math.abs(this.hostFirstLeafPosition.y - this.hostLastLeafPosition.y)
				+ Math.abs(this.parasiteFirstLeafPosition.y - this.parasiteLastLeafPosition.y);
		// if (rootXDistance >= rootYDistance) {
		if (verticalDistance >= horizontalDistance) {
			this.arrangementFlag = ARRANGEMENT_HORIZONTAL;
			if (this.hostRootPosition.x <= this.parasiteRootPosition.x) {
				this.positionFlag = POSITION_FIRST_HOST_SECOND_PARASITE;
				if (this.hostFirstLeafPosition.y > this.hostLastLeafPosition.y) {
					this.hostFlipped = true;
				} else {
					this.hostFlipped = false;
				}
				if (this.parasiteFirstLeafPosition.y >= this.parasiteLastLeafPosition.y) {
					this.parasiteFlipped = false;
				} else {
					this.parasiteFlipped = true;
				}
			} else {
				this.positionFlag = POSITION_FIRST_PARASITE_SECOND_HOST;
				if (this.hostFirstLeafPosition.y >= this.hostLastLeafPosition.y) {
					this.hostFlipped = false;
				} else {
					this.hostFlipped = true;
				}
				if (this.parasiteFirstLeafPosition.y > this.parasiteLastLeafPosition.y) {
					this.parasiteFlipped = true;
				} else {
					this.parasiteFlipped = false;
				}
			}
		} else {
			this.arrangementFlag = ARRANGEMENT_VERTICAL;
			if (this.hostRootPosition.y <= this.parasiteRootPosition.y) {
				this.positionFlag = POSITION_FIRST_HOST_SECOND_PARASITE;
				if (this.hostFirstLeafPosition.x >= this.hostLastLeafPosition.x) {
					this.hostFlipped = false;
				} else {
					this.hostFlipped = true;
				}
				if (this.parasiteFirstLeafPosition.x > this.parasiteLastLeafPosition.x) {
					this.parasiteFlipped = true;
				} else {
					this.parasiteFlipped = false;
				}
			} else {
				this.positionFlag = POSITION_FIRST_PARASITE_SECOND_HOST;
				if (this.hostFirstLeafPosition.x > this.hostLastLeafPosition.x) {
					this.hostFlipped = true;
				} else {
					this.hostFlipped = false;
				}
				if (this.parasiteFirstLeafPosition.x >= this.parasiteLastLeafPosition.x) {
					this.parasiteFlipped = false;
				} else {
					this.parasiteFlipped = true;
				}
			}
		}
	}

	/**
	 * @return the hostRootPosition
	 */
	public Point getTreeRootPosition(int tree) {
		switch (tree) {
		case InputViewerConstants.TREE_SELECTION_HOST:
			return hostRootPosition;
		case InputViewerConstants.TREE_SELECTION_PARASITE:
			return parasiteRootPosition;
		default:
			return null;
		}
	}

	/**
	 * @return the arrangementFlag
	 */
	public int getArrangementFlag() {
		return arrangementFlag;
	}

	/**
	 * @return the positionFlag
	 */
	public int getPositionFlag() {
		return positionFlag;
	}

	/**
	 * @return the hostFlipped
	 */
	public boolean isTreeFlipped(int tree) {
		switch (tree) {
		case InputViewerConstants.TREE_SELECTION_HOST:
			return hostFlipped;
		case InputViewerConstants.TREE_SELECTION_PARASITE:
			return parasiteFlipped;
		default:
			return false;
		}
	}

}
