/**
 * 
 */
package com.wieseke.cptk.input.wizards;

import org.eclipse.jface.wizard.Wizard;

import com.wieseke.cptk.common.dao.ParasiteRank;
import com.wieseke.cptk.input.dao.InputParasiteNode;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ChangeParasiteRankWizard extends Wizard {

	private ChangeParasiteRankWizardPage changeParasiteRankWizardPage;
	private InputParasiteNode node;
	private ParasiteRank rank;

	public ChangeParasiteRankWizard(InputParasiteNode node) {
		this.node = node;
	}

	
	public boolean performFinish() {
		int value1 = changeParasiteRankWizardPage.getValue1();
		int value2 = changeParasiteRankWizardPage.getValue2();
		if (value1 < 0 || value2 < 0) {
			this.changeParasiteRankWizardPage
					.setDescription("The timezone values must be greater than 0.");
			return false;
		}
		if (value1 > value2) {
			this.changeParasiteRankWizardPage
					.setDescription("The second value must be greater or equal than the first value.");
			return false;
		}
		ParasiteRank parentRank = node.getParentRank();
		if (parentRank.getRankFrom() > value1) {
			this.changeParasiteRankWizardPage
					.setDescription("The beginning of the timezone interval must be greater or equal than the beginning of the timezone interval from the parent ["
							+ parentRank.getRankFrom()
							+ ":"
							+ parentRank.getRankTo() + "].");
			return false;
		}
		if (parentRank.getRankTo() > value2) {
			this.changeParasiteRankWizardPage
					.setDescription("The ending of the timezone interval must be greater or equal than the ending of the timezone interval from the parent ["
							+ parentRank.getRankFrom()
							+ ":"
							+ parentRank.getRankTo() + "].");
			return false;
		}
		ParasiteRank childRank = node.getMinChildRank();
		if (!node.isLeaf() && childRank.getRankFrom() < value1) {
			this.changeParasiteRankWizardPage
					.setDescription("The beginning of the timezone intervall must be lower or equal to the smallest beginning of the timezone interval from the children ["
							+ childRank.getRankFrom()
							+ ":"
							+ childRank.getRankTo() + "].");
			return false;
		}
		if (!node.isLeaf() && childRank.getRankTo() < value2) {
			this.changeParasiteRankWizardPage
					.setDescription("The ending of the timezone intervall must be lower or equal to the smallest ending of the timezone interval from the children ["
							+ childRank.getRankFrom()
							+ ":"
							+ childRank.getRankTo() + "].");
			return false;
		}
		this.rank = new ParasiteRank(value1, value2);
		return true;
	}

	
	public void addPages() {
		changeParasiteRankWizardPage = new ChangeParasiteRankWizardPage(
				"Change label", node.getRank().getRankFrom(), node.getRank()
						.getRankTo());
		addPage(changeParasiteRankWizardPage);
		super.addPages();
	}

	/**
	 * @return the rank
	 */
	public ParasiteRank getRank() {
		return rank;
	}
}
