/**
 * 
 */
package com.wieseke.cptk.input.wizards;

import java.util.Random;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import com.wieseke.cptk.reconstruction.constants.StatisticConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class RandomNexusFileWizardPage2 extends WizardPage {

	private Group groupTreeParameters;
	private Composite compositeLeaves;
	private Label labelHostLeaves;
	private Label labelParasiteLeaves;
	private Spinner spinnerParasiteLeaves;
	private Spinner spinnerHostLeaves;
	private Composite compositeTreeModel;
	private Label labelTreeModel;
	private Button buttonNoAssociation;
	private Button buttonUniqueAssociations;
	private Button buttonMultiplaAssociations;
	private Group groupLeafAssociationParameters;
	private Composite compositeLeafAssociationParameters;
	private Button buttonMarkov;
	private Button buttonAldous;
	private Button buttonPDA;
	private Button buttonAge;
	private Group groupParameters;
	private Composite compositeParameters;
	private Label labelRandomSeed;
	private Text textRandomSeed;

	public RandomNexusFileWizardPage2() {
		super("WizardRandomNexusFileCreationPage2");
		setTitle("Random Nexus File");
		setDescription("Creates a Random Nexus File ressource");
	}

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.FILL);
		GridLayout layout = new GridLayout();
		layout.makeColumnsEqualWidth = true;
		composite.setLayout(layout);

		groupTreeParameters = new Group(composite, SWT.NONE);
		GridLayout groupTreeParametersLayout = new GridLayout();
		groupTreeParameters.setLayout(groupTreeParametersLayout);
		GridData groupTreeParametersLData = new GridData();
		groupTreeParametersLData.grabExcessHorizontalSpace = true;
		groupTreeParametersLData.horizontalAlignment = GridData.FILL;
		groupTreeParameters.setLayoutData(groupTreeParametersLData);
		groupTreeParameters.setText("Tree Parameters");

		compositeLeaves = new Composite(groupTreeParameters, SWT.NONE);
		GridLayout compositeLeavesLayout = new GridLayout();
		compositeLeavesLayout.makeColumnsEqualWidth = true;
		compositeLeavesLayout.numColumns = 2;
		compositeLeaves.setLayout(compositeLeavesLayout);
		GridData compositeLeavesLData = new GridData();
		compositeLeavesLData.horizontalAlignment = GridData.FILL;
		compositeLeaves.setLayoutData(compositeLeavesLData);

		labelHostLeaves = new Label(compositeLeaves, SWT.NONE);
		GridData labelHostLeavesLData = new GridData();
		labelHostLeavesLData.horizontalAlignment = GridData.FILL;
		labelHostLeavesLData.grabExcessHorizontalSpace = true;
		labelHostLeaves.setLayoutData(labelHostLeavesLData);
		labelHostLeaves.setText("Number of host leaves");

		labelParasiteLeaves = new Label(compositeLeaves, SWT.NONE);
		GridData labelParasiteLeavesLData = new GridData();
		labelParasiteLeavesLData.horizontalAlignment = GridData.FILL;
		labelParasiteLeaves.setLayoutData(labelParasiteLeavesLData);
		labelParasiteLeavesLData.grabExcessHorizontalSpace = true;
		labelParasiteLeaves.setText("Number of parasite leaves");

		spinnerHostLeaves = new Spinner(compositeLeaves, SWT.BORDER);
		GridData spinnerHostLeavesLData = new GridData();
		spinnerHostLeavesLData.horizontalAlignment = GridData.FILL;
		spinnerHostLeavesLData.grabExcessHorizontalSpace = true;
		spinnerHostLeaves.setLayoutData(spinnerHostLeavesLData);
		spinnerHostLeaves.setMinimum(1);
		spinnerHostLeaves.setMaximum(2500);
		spinnerHostLeaves.setSelection(10);

		spinnerParasiteLeaves = new Spinner(compositeLeaves, SWT.BORDER);
		GridData spinnerParasiteLeavesLData = new GridData();
		spinnerParasiteLeavesLData.horizontalAlignment = GridData.FILL;
		spinnerParasiteLeavesLData.grabExcessHorizontalSpace = true;
		spinnerParasiteLeaves.setLayoutData(spinnerParasiteLeavesLData);
		spinnerParasiteLeaves.setMinimum(1);
		spinnerParasiteLeaves.setMaximum(2500);
		spinnerParasiteLeaves.setSelection(10);

		compositeTreeModel = new Composite(groupTreeParameters, SWT.NONE);
		GridLayout compositeBetaSplittingModelLayout = new GridLayout();
		compositeBetaSplittingModelLayout.makeColumnsEqualWidth = true;
		compositeTreeModel.setLayout(compositeBetaSplittingModelLayout);
		GridData compositeBetaSplittingModelLData = new GridData();
		compositeBetaSplittingModelLData.horizontalAlignment = GridData.FILL;
		compositeBetaSplittingModelLData.grabExcessHorizontalSpace = true;
		compositeTreeModel.setLayoutData(compositeBetaSplittingModelLData);

		labelTreeModel = new Label(compositeTreeModel, SWT.NONE);
		labelTreeModel.setText("Tree Model");

		buttonMarkov = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		buttonMarkov.setText("Markov (\u03b2=0)");

		buttonAldous = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		buttonAldous.setText("Aldous (\u03b2=-1)");

		buttonPDA = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		buttonPDA.setText("PDA (\u03b2=-1.5)");

		buttonAge = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		buttonAge.setText("Age");
		buttonAge.setSelection(true);

		groupLeafAssociationParameters = new Group(composite, SWT.NONE);
		FillLayout groupLeafAssociationParametersLayout = new FillLayout(
				org.eclipse.swt.SWT.HORIZONTAL);
		groupLeafAssociationParameters
				.setLayout(groupLeafAssociationParametersLayout);
		GridData groupLeafAssociationParametersLData = new GridData();
		groupLeafAssociationParametersLData.horizontalAlignment = GridData.FILL;
		groupLeafAssociationParametersLData.grabExcessHorizontalSpace = true;
		groupLeafAssociationParameters
				.setLayoutData(groupLeafAssociationParametersLData);
		groupLeafAssociationParameters.setText("Leaf Association Parameters");

		compositeLeafAssociationParameters = new Composite(
				groupLeafAssociationParameters, SWT.NONE);
		GridLayout compositeLeafAssociationParametersLayout = new GridLayout();
		compositeLeafAssociationParametersLayout.makeColumnsEqualWidth = true;
		compositeLeafAssociationParameters
				.setLayout(compositeLeafAssociationParametersLayout);

		buttonNoAssociation = new Button(compositeLeafAssociationParameters,
				SWT.RADIO | SWT.LEFT);
		buttonNoAssociation.setText("Create no associations");

		buttonUniqueAssociations = new Button(
				compositeLeafAssociationParameters, SWT.RADIO | SWT.LEFT);
		buttonUniqueAssociations
				.setText("Create random associations with unique host assignments");

		buttonMultiplaAssociations = new Button(
				compositeLeafAssociationParameters, SWT.RADIO | SWT.LEFT);
		buttonMultiplaAssociations
				.setText("Create random associations with multiple host assignments");
		buttonMultiplaAssociations.setSelection(true);

		groupParameters = new Group(composite, SWT.NONE);
		FillLayout groupParametersLayout = new FillLayout(
				org.eclipse.swt.SWT.HORIZONTAL);
		groupParameters.setLayout(groupParametersLayout);
		GridData groupParametersLData = new GridData();
		groupParametersLData.horizontalAlignment = GridData.FILL;
		groupParametersLData.grabExcessHorizontalSpace = true;
		groupParameters.setLayoutData(groupParametersLData);
		groupParameters.setText("Parameters");

		compositeParameters = new Composite(groupParameters, SWT.NONE);
		GridLayout compositeParametersLayout = new GridLayout();
		compositeParametersLayout.makeColumnsEqualWidth = true;
		compositeParametersLayout.numColumns = 2;
		compositeParameters.setLayout(compositeParametersLayout);

		labelRandomSeed = new Label(compositeParameters, SWT.NONE);
		labelRandomSeed.setText("Random Seed Value");

		textRandomSeed = new Text(compositeParameters, SWT.BORDER);
		textRandomSeed.setText("auto");
		GridData textRandomSeedLData = new GridData();
		textRandomSeedLData.horizontalAlignment = GridData.FILL;
		textRandomSeedLData.grabExcessHorizontalSpace = true;
		textRandomSeed.setLayoutData(textRandomSeedLData);

		composite.layout();
		setControl(composite);
	}

	public int getHostLeaves() {
		return this.spinnerHostLeaves.getSelection();
	}

	public int getParasiteLeaves() {
		return this.spinnerParasiteLeaves.getSelection();
	}

	public String getModel() {
		if (this.buttonMarkov.getSelection())
			return StatisticConstants.MODEL_MARKOV;
		if (this.buttonAldous.getSelection())
			return StatisticConstants.MODEL_ALDOUS;
		if (this.buttonPDA.getSelection())
			return StatisticConstants.MODEL_PDA;
		if (this.buttonAge.getSelection())
			return StatisticConstants.MODEL_AGE;
		return StatisticConstants.MODEL_AGE;
	}

	public int getAssociation() {
		if (this.buttonNoAssociation.getSelection())
			return StatisticConstants.ASSOCIATION_NONE;
		if (this.buttonUniqueAssociations.getSelection())
			return StatisticConstants.ASSOCIATION_UNIQUE;
		if (this.buttonMultiplaAssociations.getSelection())
			return StatisticConstants.ASSOCIATION_MULTIPLE;
		return StatisticConstants.ASSOCIATION_MULTIPLE;
	}

	public Random getRandom() {
		Random rand = new Random();
		{
			String seed = this.textRandomSeed.getText();
			if (seed != null && !seed.equals("")) {
				try {
					rand = new Random(Long.parseLong(seed));
				} catch (Exception e) {
				}
			}
		}
		return rand;
	}
}