/**
 * 
 */
package com.wieseke.cptk.input.wizards;

import java.util.Random;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import com.wieseke.cptk.reconstruction.constants.StatisticConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class RandomSimulationNexusFileWizardPage2 extends WizardPage {

	private Group groupTreeParameters;
	private Composite compositeLeaves;
	private Label labelTotalLeaves;
	private Spinner spinnerTotalLeaves;
	private Composite compositeTreeModel;
	private Label labelTreeModel;
	private Button buttonMarkov;
	// private Button buttonAldous;
	// private Button buttonPDA;
	private Button buttonAge;
	private Group groupParameters;
	private Composite compositeParameters;
	private Label labelHostChoosingProbability;
	private Text textHostChoosingProbability;
	private Label labelCospeciationProbability;
	private Text textCospeciationProbability;
	private Label labelSwitchingProbability;
	private Text textSwitchingProbability;
	private Label labelPreferentialHostSwitchAlpha;
	private Text textPreferentialHostSwitchAlpha;
	private Label labelMultihostProbability;
	private Text textMultihostProbability;
	private Label labelUseMergedNodeList;
	private Button buttonUseMergedNodeList;
	private Label labelRandomSeed;
	private Text textRandomSeed;

	public RandomSimulationNexusFileWizardPage2() {
		super("WizardRandomNexusFileCreationPage2");
		setTitle("Random Simulation Nexus File");
		setDescription("Creates a Random Simulation Nexus File ressource");
	}

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.FILL);
		GridLayout layout = new GridLayout();
		layout.makeColumnsEqualWidth = true;
		composite.setLayout(layout);

		groupTreeParameters = new Group(composite, SWT.NONE);
		GridLayout groupTreeParametersLayout = new GridLayout();
		groupTreeParameters.setLayout(groupTreeParametersLayout);
		GridData groupTreeParametersLData = new GridData();
		groupTreeParametersLData.grabExcessHorizontalSpace = true;
		groupTreeParametersLData.horizontalAlignment = GridData.FILL;
		groupTreeParameters.setLayoutData(groupTreeParametersLData);
		groupTreeParameters.setText("Tree Parameters");

		compositeLeaves = new Composite(groupTreeParameters, SWT.NONE);
		GridLayout compositeLeavesLayout = new GridLayout();
		compositeLeavesLayout.makeColumnsEqualWidth = true;
		compositeLeavesLayout.numColumns = 2;
		compositeLeaves.setLayout(compositeLeavesLayout);
		GridData compositeLeavesLData = new GridData();
		compositeLeavesLData.horizontalAlignment = GridData.FILL;
		compositeLeaves.setLayoutData(compositeLeavesLData);

		labelTotalLeaves = new Label(compositeLeaves, SWT.NONE);
		GridData labelHostLeavesLData = new GridData();
		labelHostLeavesLData.horizontalAlignment = GridData.FILL;
		labelHostLeavesLData.grabExcessHorizontalSpace = true;
		labelTotalLeaves.setLayoutData(labelHostLeavesLData);
		labelTotalLeaves.setText("Number of leaves");

		spinnerTotalLeaves = new Spinner(compositeLeaves, SWT.BORDER);
		GridData spinnerHostLeavesLData = new GridData();
		spinnerHostLeavesLData.horizontalAlignment = GridData.FILL;
		spinnerHostLeavesLData.grabExcessHorizontalSpace = true;
		spinnerTotalLeaves.setLayoutData(spinnerHostLeavesLData);
		spinnerTotalLeaves.setMinimum(1);
		spinnerTotalLeaves.setMaximum(1000);
		spinnerTotalLeaves.setSelection(10);

		compositeTreeModel = new Composite(groupTreeParameters, SWT.NONE);
		GridLayout compositeTreeModelLayout = new GridLayout();
		compositeTreeModelLayout.makeColumnsEqualWidth = true;
		compositeTreeModel.setLayout(compositeTreeModelLayout);
		GridData compositeTreeModelLData = new GridData();
		compositeTreeModelLData.horizontalAlignment = GridData.FILL;
		compositeTreeModelLData.grabExcessHorizontalSpace = true;
		compositeTreeModel.setLayoutData(compositeTreeModelLData);

		labelTreeModel = new Label(compositeTreeModel, SWT.NONE);
		labelTreeModel.setText("Beta Splitting / Age Model");

		buttonMarkov = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		buttonMarkov.setText("Markov (\u03b2=0)");

		// buttonAldous = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		// buttonAldous.setText("Aldous (\u03b2=-1)");
		//
		// buttonPDA = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		// buttonPDA.setText("PDA (\u03b2=-1.5)");

		buttonAge = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		buttonAge.setText("Age");
		buttonAge.setSelection(true);

		groupParameters = new Group(composite, SWT.NONE);
		FillLayout groupParametersLayout = new FillLayout(
				org.eclipse.swt.SWT.HORIZONTAL);
		groupParameters.setLayout(groupParametersLayout);
		GridData groupParametersLData = new GridData();
		groupParametersLData.horizontalAlignment = GridData.FILL;
		groupParametersLData.grabExcessHorizontalSpace = true;
		groupParameters.setLayoutData(groupParametersLData);
		groupParameters.setText("Parameters");

		compositeParameters = new Composite(groupParameters, SWT.NONE);
		GridLayout compositeParametersLayout = new GridLayout();
		compositeParametersLayout.makeColumnsEqualWidth = true;
		compositeParametersLayout.numColumns = 2;
		compositeParameters.setLayout(compositeParametersLayout);

		labelHostChoosingProbability = new Label(compositeParameters, SWT.NONE);
		labelHostChoosingProbability.setText("Host choosing probability");
		textHostChoosingProbability = new Text(compositeParameters, SWT.BORDER);
		textHostChoosingProbability.setText("0.5");
		GridData textHostChoosingProbabilityLData = new GridData();
		textHostChoosingProbabilityLData.horizontalAlignment = GridData.FILL;
		textHostChoosingProbabilityLData.grabExcessHorizontalSpace = true;
		textHostChoosingProbability
				.setLayoutData(textHostChoosingProbabilityLData);

		labelCospeciationProbability = new Label(compositeParameters, SWT.NONE);
		labelCospeciationProbability
				.setText("Cospeciation probability (1 - Sorting)");
		textCospeciationProbability = new Text(compositeParameters, SWT.BORDER);
		textCospeciationProbability.setText("0.5");
		GridData textCospeciationProbabilityLData = new GridData();
		textCospeciationProbabilityLData.horizontalAlignment = GridData.FILL;
		textCospeciationProbabilityLData.grabExcessHorizontalSpace = true;
		textCospeciationProbability
				.setLayoutData(textCospeciationProbabilityLData);

		labelSwitchingProbability = new Label(compositeParameters, SWT.NONE);
		labelSwitchingProbability
				.setText("Switching probability (1 - Duplication)");
		textSwitchingProbability = new Text(compositeParameters, SWT.BORDER);
		textSwitchingProbability.setText("0.25");
		GridData textSwitchingProbabilityLData = new GridData();
		textSwitchingProbabilityLData.horizontalAlignment = GridData.FILL;
		textSwitchingProbabilityLData.grabExcessHorizontalSpace = true;
		textSwitchingProbability.setLayoutData(textSwitchingProbabilityLData);

		labelPreferentialHostSwitchAlpha = new Label(compositeParameters,
				SWT.NONE);
		labelPreferentialHostSwitchAlpha
				.setText("Preferential host switching alpha");
		textPreferentialHostSwitchAlpha = new Text(compositeParameters,
				SWT.BORDER);
		textPreferentialHostSwitchAlpha.setText("0");
		GridData textPreferentialHostSwitchAlphaLData = new GridData();
		textPreferentialHostSwitchAlphaLData.horizontalAlignment = GridData.FILL;
		textPreferentialHostSwitchAlphaLData.grabExcessHorizontalSpace = true;
		textPreferentialHostSwitchAlpha
				.setLayoutData(textPreferentialHostSwitchAlphaLData);

		labelMultihostProbability = new Label(compositeParameters, SWT.NONE);
		labelMultihostProbability.setText("Multi-host probability");
		textMultihostProbability = new Text(compositeParameters, SWT.BORDER);
		textMultihostProbability.setText("0");
		GridData textMultihostProbabilityLData = new GridData();
		textMultihostProbabilityLData.horizontalAlignment = GridData.FILL;
		textMultihostProbabilityLData.grabExcessHorizontalSpace = true;
		textMultihostProbability.setLayoutData(textMultihostProbabilityLData);

		labelUseMergedNodeList = new Label(compositeParameters, SWT.NONE);
		labelUseMergedNodeList.setText("");
		buttonUseMergedNodeList = new Button(compositeParameters, SWT.CHECK
				| SWT.LEFT);
		buttonUseMergedNodeList.setText("Use merged host/parasite list");
		buttonUseMergedNodeList.setSelection(false);

		labelRandomSeed = new Label(compositeParameters, SWT.NONE);
		labelRandomSeed.setText("Random Seed Value");
		textRandomSeed = new Text(compositeParameters, SWT.BORDER);
		textRandomSeed.setText("auto");
		GridData textRandomSeedLData = new GridData();
		textRandomSeedLData.horizontalAlignment = GridData.FILL;
		textRandomSeedLData.grabExcessHorizontalSpace = true;
		textRandomSeed.setLayoutData(textRandomSeedLData);

		composite.layout();
		setControl(composite);
	}

	public int getHostLeaves() {
		return this.spinnerTotalLeaves.getSelection();
	}

	public String getModel() {
		if (this.buttonMarkov.getSelection())
			return StatisticConstants.MODEL_MARKOV;
		// if (this.buttonAldous.getSelection())
		// return StatisticConstants.MODEL_ALDOUS;
		// if (this.buttonPDA.getSelection())
		// return StatisticConstants.MODEL_PDA;
		if (this.buttonAge.getSelection())
			return StatisticConstants.MODEL_AGE;
		return StatisticConstants.MODEL_AGE;
	}

	public double getHostChoosingProbability() {
		return Double.parseDouble(textHostChoosingProbability.getText());
	}

	public double getCospeciationProbability() {
		return Double.parseDouble(textCospeciationProbability.getText());
	}

	public double getSwitchingProbability() {
		return Double.parseDouble(textSwitchingProbability.getText());
	}

	public double getPreferentialHostSwitchAlpha() {
		return Double.parseDouble(textPreferentialHostSwitchAlpha.getText());
	}

	public double getExtincrionProbability() {
		return Double.parseDouble(textMultihostProbability.getText());
	}

	public boolean isUseMergedNodeList() {
		return buttonUseMergedNodeList.getSelection();
	}

	public Random getRandom() {
		Random rand = new Random();
		{
			String seed = this.textRandomSeed.getText();
			if (seed != null && !seed.equals("")) {
				try {
					rand = new Random(Long.parseLong(seed));
				} catch (Exception e) {
				}
			}
		}
		return rand;
	}

}