/**
 * 
 */
package com.wieseke.cptk.main;

import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStreamWriter;
import java.util.Map;

import org.apache.commons.io.FileUtils;

import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.common.dao.ReconstructionCosts;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.format.nexus.NexusDeserializer;
import com.wieseke.cptk.output.dao.OutputCophylogeny;
import com.wieseke.cptk.reconstruction.dao.ReconstructionCophylogeny;

/**
 * @author Nicolas Wieseke
 * 
 */
public class Reconstruction {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// double stdCosts[] = { 0d, 0d, 0d, 0d }; // all reconstructions
		double stdCosts[] = { -2d, 1d, 2d, 3d }; // -1 1 1 1 in treemap cost
		double stdCosts2[] = { 0d, 1d, 2d, 3d }; // 0 1 1 1 in treemap cost
		if (args.length == 0 || args[0].equalsIgnoreCase("-h")
				|| args[0].equalsIgnoreCase("-help")) {
			printHelp();
			return;
		}
		try {
			String filename = args[0];
			NexusDeserializer nd = new NexusDeserializer();
			InputCophylogeny cophylogeny = nd.deserialize(new FileInputStream(
					filename), filename);
			cophylogeny.deleteNoneLeafAssociations(cophylogeny
					.getParasiteRoot());
			ReconstructionCophylogeny reconstructionCophylogeny;

			if (args[1].equals("0") || args[1].equals("all")) {
				// Reconstruction with standard costs
				ReconstructionCosts costs = new ReconstructionCosts(stdCosts);
				Map<String, String> options = InputCophylogeny
						.createStandardOptions();
				options.put(OptionsConstants.AUTOMATIC_COSTS_ID,
						OptionsConstants.AUTOMATIC_COSTS_0_ID);
				reconstructionCophylogeny = new ReconstructionCophylogeny(
						cophylogeny, costs, options);
				// OutputStreamWriter oswStdLog = new OutputStreamWriter(
				// FileUtils.openOutputStream(new File(filename + ".std.log")));
				// oswStdLog
				// .write("Counter\tQuality\tTotalCost\tTotalEvents\tCoCost\tCoEvents\tCoProb\tCoFreq\tSoCost\tSoEvents\tSoProb\tSoFreq\tDuCost\tDuEvents\tDuProb\tDuFreq\tHsCost\tHsEvents\tHsProb\tHsFreq\tRootMapping\n");
				// reconstructionCophylogeny.compute(oswStdLog);
				// oswStdLog.close();
				long time = System.currentTimeMillis();
				reconstructionCophylogeny.compute(null);
				time = System.currentTimeMillis() - time;
				OutputCophylogeny outputCophylogeny = new OutputCophylogeny(
						reconstructionCophylogeny);
				ReconstructionHelper rh = new ReconstructionHelper(
						outputCophylogeny);
				OutputStreamWriter oswStdReco = new OutputStreamWriter(
						FileUtils.openOutputStream(new File(filename
								+ ".std1.reco")));
				oswStdReco.write("Duration in milliseconds:\n" + time + "\n");
				rh.computeAssociationLists(oswStdReco);
				oswStdReco.close();
			}

			if (args[1].equals("1") || args[1].equals("all")) {
				// Reconstruction with standard costs
				ReconstructionCosts costs = new ReconstructionCosts(stdCosts2);
				Map<String, String> options = InputCophylogeny
						.createStandardOptions();
				options.put(OptionsConstants.AUTOMATIC_COSTS_ID,
						OptionsConstants.AUTOMATIC_COSTS_0_ID);
				reconstructionCophylogeny = new ReconstructionCophylogeny(
						cophylogeny, costs, options);
				// OutputStreamWriter oswStdLog = new OutputStreamWriter(
				// FileUtils.openOutputStream(new File(filename + ".std.log")));
				// oswStdLog
				// .write("Counter\tQuality\tTotalCost\tTotalEvents\tCoCost\tCoEvents\tCoProb\tCoFreq\tSoCost\tSoEvents\tSoProb\tSoFreq\tDuCost\tDuEvents\tDuProb\tDuFreq\tHsCost\tHsEvents\tHsProb\tHsFreq\tRootMapping\n");
				// reconstructionCophylogeny.compute(oswStdLog);
				// oswStdLog.close();
				long time = System.currentTimeMillis();
				reconstructionCophylogeny.compute(null);
				time = System.currentTimeMillis() - time;
				OutputCophylogeny outputCophylogeny = new OutputCophylogeny(
						reconstructionCophylogeny);
				ReconstructionHelper rh = new ReconstructionHelper(
						outputCophylogeny);
				OutputStreamWriter oswStdReco = new OutputStreamWriter(
						FileUtils.openOutputStream(new File(filename
								+ ".std2.reco")));
				oswStdReco.write("Duration in milliseconds:\n" + time + "\n");
				rh.computeAssociationLists(oswStdReco);
				oswStdReco.close();
			}

			if (args[1].equals("2") || args[1].equals("all")) {
				// Reconstruction with adapted costs
				ReconstructionCosts costs = new ReconstructionCosts(stdCosts);
				Map<String, String> options = InputCophylogeny
						.createStandardOptions();
				options.put(OptionsConstants.AUTOMATIC_COSTS_ID,
						OptionsConstants.AUTOMATIC_COSTS_1_ID);
				options.put(OptionsConstants.AUTOMATIC_METHOD_ID,
						OptionsConstants.AUTOMATIC_METHOD_2_ID);
				options.put(OptionsConstants.RANDOM_CYCLES_ID, "1000");
				reconstructionCophylogeny = new ReconstructionCophylogeny(
						cophylogeny, costs, options);
				// OutputStreamWriter oswPALog = new OutputStreamWriter(
				// FileUtils.openOutputStream(new File(filename + ".pa.log")));
				// oswPALog.write("Counter\tQuality\tTotalCost\tTotalEvents\tCoCost\tCoEvents\tCoProb\tCoFreq\tSoCost\tSoEvents\tSoProb\tSoFreq\tDuCost\tDuEvents\tDuProb\tDuFreq\tHsCost\tHsEvents\tHsProb\tHsFreq\tRootMapping\n");
				// reconstructionCophylogeny.compute(oswPALog);
				// oswPALog.close();
				long time = System.currentTimeMillis();
				reconstructionCophylogeny.compute(null);
				time = System.currentTimeMillis() - time;
				OutputCophylogeny outputCophylogeny = new OutputCophylogeny(
						reconstructionCophylogeny);
				ReconstructionHelper rh = new ReconstructionHelper(
						outputCophylogeny);
				OutputStreamWriter oswPaReco = new OutputStreamWriter(
						FileUtils.openOutputStream(new File(filename
								+ ".pa.reco")));
				oswPaReco.write("Duration in milliseconds:\n" + time + "\n");
				rh.computeAssociationLists(oswPaReco);
				oswPaReco.close();
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * prints the help information
	 * */
	static public void printHelp() {
		System.out
				.println("\nSyntax:\n"
						+ "java -cp .:./* com.wieseke.cptk.main.Reconstruction [filename.nex] [0|1|2|all]"
						+ "\n\nFor the java -cp parameter use ';' as classpath separanor on Windows machines (not ':').");
	}
}
