package com.wieseke.cptk.main;

import java.io.IOException;
import java.io.OutputStreamWriter;

import com.wieseke.cptk.common.util.NodeUtils;
import com.wieseke.cptk.output.dao.OutputCophylogeny;
import com.wieseke.cptk.output.dao.OutputHostNode;
import com.wieseke.cptk.output.dao.OutputNode;
import com.wieseke.cptk.output.dao.OutputParasiteNode;
import com.wieseke.cptk.reconstruction.dao.EventType;
import com.wieseke.cptk.reconstruction.dao.ReconstructionChildMapping;

public class ReconstructionHelper {

	private OutputCophylogeny outputCophylogeny;

	public ReconstructionHelper(OutputCophylogeny outputCophylogeny) {
		this.outputCophylogeny = outputCophylogeny;
	}

	public void computeAssociationLists(OutputStreamWriter oswReco) {
		computeAssociationLists(oswReco, null);
	}

	public void computeAssociationLists(OutputStreamWriter oswReco,
			OutputStreamWriter oswCosp) {
		int i = 0;
		do {
			// if (i == 1000) {
			// break;
			// }
			if (!outputCophylogeny.isValid()) {
				continue;
			}
			String output = "EVENTS\n";
			EventType events = outputCophylogeny.getSolution().getEvents();
			output += events.getCospeciations() + "\t" + events.getSortings()
					+ "\t" + events.getDuplications() + "\t"
					+ events.getHostswitches() + "\n";

			output += "QUALITY\n";
			output += outputCophylogeny.getSolution().getQuality()
					.toPlainString()
					+ "\n";

			output += "COSTS\n";
			output += outputCophylogeny.getSolution().getCosts().toString()
					+ "\n";

			output += "RECONSTRUCTION\n";
			output += computeSpeciationAssociationListsForParasiteExt(outputCophylogeny
					.getParasiteRoot());
			output += "\n";

			String outputCosp = "";
			if (oswCosp != null) {
				int[][] cospeciations = new int[outputCophylogeny.getHostRoot()
						.getChildrenCount() + 1][outputCophylogeny
						.getParasiteRoot().getChildrenCount() + 1];
				computeCospeciationAssociationListsForParasite(
						outputCophylogeny.getParasiteRoot(), cospeciations);
				String[] hostNames = new String[outputCophylogeny.getHostRoot()
						.getChildrenCount() + 1];
				String[] parasiteNames = new String[outputCophylogeny
						.getParasiteRoot().getChildrenCount() + 1];
				getNames(outputCophylogeny.getHostRoot(), hostNames);
				getNames(outputCophylogeny.getParasiteRoot(), parasiteNames);
				outputCosp += "COSPECIATIONS\n";
				for (int x = 0; x < hostNames.length; x++) {
					for (int y = 0; y < parasiteNames.length; y++) {
						if (cospeciations[x][y] > 0) {
							outputCosp += parasiteNames[y] + "\t"
									+ hostNames[x] + "\t" + cospeciations[x][y]
									+ "\n";
						}
					}
				}
			}

			try {
				oswReco.write(output);
				oswReco.flush();
				if (oswCosp != null) {
					oswCosp.write(outputCosp);
					oswCosp.flush();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
			i++;
		} while (outputCophylogeny.nextReconstruction(false, false));
	}

	public void computeAssociationListsViroBonn(OutputStreamWriter osw) {
		int[][] cospeciations = new int[outputCophylogeny.getHostRoot()
				.getChildrenCount() + 1][outputCophylogeny.getParasiteRoot()
				.getChildrenCount() + 1];
		int i = 0;
		do {
			if (!outputCophylogeny.isValid()) {
				continue;
			}
			computeCospeciationAssociationListsForParasite(
					outputCophylogeny.getParasiteRoot(), cospeciations);
			i++;
		} while (outputCophylogeny.nextReconstruction(false, false));
		String output = "#SOLUTIONS:\n";
		output += i + "\n";
		String[] hostNames = new String[outputCophylogeny.getHostRoot()
				.getChildrenCount() + 1];
		String[] parasiteNames = new String[outputCophylogeny.getParasiteRoot()
				.getChildrenCount() + 1];
		getNames(outputCophylogeny.getHostRoot(), hostNames);
		getNames(outputCophylogeny.getParasiteRoot(), parasiteNames);
		output += "COSPECIATIONS\n";
		for (int x = 0; x < hostNames.length; x++) {
			for (int y = 0; y < parasiteNames.length; y++) {
				if (cospeciations[x][y] > 0) {
					output += parasiteNames[y] + "\t" + hostNames[x] + "\t"
							+ cospeciations[x][y] + "\n";
				}
			}
		}
		try {
			osw.write(output);
			osw.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void getNames(OutputNode node, String[] names) {
		names[node.getNumber()] = node.getLabel();
		for (OutputNode child : node.getChildren()) {
			getNames(child, names);
		}

	}

	private String computeAssociationListsForParasite(OutputParasiteNode node) {
		String output = "";

		OutputHostNode tempAssociationHostNode = node.getAssociation();
		OutputParasiteNode parent = node.getParent();
		if (parent != null) {
			ReconstructionChildMapping mapping = node.getMapping();
			OutputHostNode parentAssociationHostNode = parent.getAssociation();
			if (mapping.isHostswitch()) {
				while (tempAssociationHostNode.getNumber() >= node
						.getLandingNode().getNumber()) {
					output += node.getLabel() + "\t"
							+ tempAssociationHostNode.getLabel() + "\n";
					tempAssociationHostNode = tempAssociationHostNode
							.getParent();
					if (tempAssociationHostNode == null) {
						throw new RuntimeException();
					}
				}
				tempAssociationHostNode = node.getTakeofNode();
				if (tempAssociationHostNode.getNumber() == parentAssociationHostNode
						.getNumber()) {
					for (OutputParasiteNode child : node.getChildren()) {
						output += computeAssociationListsForParasite(child);
					}
					return output;
				}
			}
			while (tempAssociationHostNode != null
					&& tempAssociationHostNode.getNumber() > parentAssociationHostNode
							.getNumber()) {
				output += node.getLabel() + "\t"
						+ tempAssociationHostNode.getLabel() + "\n";
				tempAssociationHostNode = tempAssociationHostNode.getParent();
				if (tempAssociationHostNode == null) {
					throw new RuntimeException();
				}
			}
			if (!parent.isCospeciation(true)) {
				output += node.getLabel() + "\t"
						+ tempAssociationHostNode.getLabel() + "\n";
			}
		} else {
			output += node.getLabel() + "\t"
					+ tempAssociationHostNode.getLabel() + "\n";
		}

		for (OutputParasiteNode child : node.getChildren()) {
			output += computeAssociationListsForParasite(child);
		}

		return output;
	}

	private String computeSpeciationAssociationListsForParasite(
			OutputParasiteNode node) {
		String output = "";

		output += node.getLabel() + "\t" + node.getAssociation().getLabel()
				+ "\n";
		for (OutputParasiteNode child : node.getChildren()) {
			output += computeSpeciationAssociationListsForParasite(child);
		}

		return output;
	}

	private String computeSpeciationAssociationListsForParasiteExt(
			OutputParasiteNode node) {
		String output = "";

		output += node.getLabel() + "\t" + node.getAssociation().getLabel();
		if (node.getDirectEvents().getCospeciations() > 0) {
			output += " C";
		}
		if (node.getDirectEvents().getDuplications() > 0) {
			output += " D";
		}
		if (node.getDirectEvents().getHostswitches() > 0) {
			output += " H";
			for (OutputParasiteNode child : node.getChildren()) {
				OutputHostNode childHost = child.getAssociation();
				if (!childHost.equals(node.getAssociation())
						&& !NodeUtils.isChild(childHost, node.getAssociation())) {
					output += "(" + child.getLabel() + ":"
							+ childHost.getLabel() + ")";
				}
			}
		}
		output += "\n";
		for (OutputParasiteNode child : node.getChildren()) {
			output += computeSpeciationAssociationListsForParasiteExt(child);
		}

		return output;
	}

	private String computeCospeciationAssociationListsForParasite(
			OutputParasiteNode node, int[][] cospeciations) {
		String output = "";

		if (node.getDirectEvents().getCospeciations() > 0) {
			cospeciations[node.getAssociation().getNumber()][node.getNumber()]++;
		}
		for (OutputParasiteNode child : node.getChildren()) {
			computeCospeciationAssociationListsForParasite(child, cospeciations);
		}

		return output;
	}

}
