/**
 * 
 */
package com.wieseke.cptk.output.model;

import com.wieseke.cptk.output.dao.OutputHostNode;
import com.wieseke.cptk.output.dao.OutputNode;
import com.wieseke.cptk.output.dao.OutputParasiteNode;
import com.wieseke.cptk.reconstruction.dao.EventType;
import com.wieseke.cptk.reconstruction.dao.ReconstructionElement;

/**
 * @author Nic
 * 
 */
public class OutputNodeProperties {

	private String nodeType;
	private String label;
	private String rank;
	private String hostMapping;
	private String cost;
	private String cospeciations;
	private String sortings;
	private String duplications;
	private String hostswitches;
	private String cospeciationsSingle;
	private String sortingsSingle;
	private String duplicationsSingle;
	private String hostswitchesSingle;

	public OutputNodeProperties(OutputNode node) {
		this.nodeType = "";
		this.label = node.getLabel();
		this.rank = node.getRankText();
		this.hostMapping = "";
		this.cost = "";
		this.cospeciations = "";
		this.sortings = "";
		this.duplications = "";
		this.hostswitches = "";
		this.cospeciationsSingle = "";
		this.sortingsSingle = "";
		this.duplicationsSingle = "";
		this.hostswitchesSingle = "";
		if (node instanceof OutputHostNode) {
			this.nodeType = "host";
		} else if (node instanceof OutputParasiteNode) {
			this.nodeType = "parasite";
			OutputParasiteNode parasiteNode = (OutputParasiteNode) node;
			ReconstructionElement reconstruction = parasiteNode
					.getReconstruction();
			if (reconstruction != null) {
				this.hostMapping = parasiteNode.getHostMapping();
				this.cost = reconstruction.getCosts().toPlainString();
				EventType derivedEvents = parasiteNode.getDerivedEvents();
				if (derivedEvents != null) {
					this.cospeciations = String.valueOf(derivedEvents
							.getCospeciations());
					this.sortings = String.valueOf(derivedEvents.getSortings());
					this.duplications = String.valueOf(derivedEvents
							.getDuplications());
					this.hostswitches = String.valueOf(derivedEvents
							.getHostswitches());
				}
				EventType directEvents = parasiteNode.getDirectEvents();
				if (directEvents != null) {
					this.cospeciationsSingle = String.valueOf(directEvents
							.getCospeciations());
					this.sortingsSingle = String.valueOf(directEvents
							.getSortings());
					this.duplicationsSingle = String.valueOf(directEvents
							.getDuplications());
					this.hostswitchesSingle = String.valueOf(directEvents
							.getHostswitches());
				}
			}
		}
	}

	/**
	 * @return the nodeType
	 */
	public String getNodeType() {
		return nodeType;
	}

	/**
	 * @param nodeType
	 *            the nodeType to set
	 */
	public void setNodeType(String nodeType) {
		this.nodeType = nodeType;
	}

	/**
	 * @return the label
	 */
	public String getLabel() {
		return label;
	}

	/**
	 * @param label
	 *            the label to set
	 */
	public void setLabel(String label) {
		this.label = label;
	}

	/**
	 * @return the rank
	 */
	public String getRank() {
		return rank;
	}

	/**
	 * @param rank
	 *            the rank to set
	 */
	public void setRank(String rank) {
		this.rank = rank;
	}

	/**
	 * @return the cost
	 */
	public String getCost() {
		return cost;
	}

	/**
	 * @param cost
	 *            the cost to set
	 */
	public void setCost(String cost) {
		this.cost = cost;
	}

	/**
	 * @return the cospeciations
	 */
	public String getCospeciations() {
		return cospeciations;
	}

	/**
	 * @param cospeciations
	 *            the cospeciations to set
	 */
	public void setCospeciations(String cospeciations) {
		this.cospeciations = cospeciations;
	}

	/**
	 * @return the sortings
	 */
	public String getSortings() {
		return sortings;
	}

	/**
	 * @param sortings
	 *            the sortings to set
	 */
	public void setSortings(String sortings) {
		this.sortings = sortings;
	}

	/**
	 * @return the duplications
	 */
	public String getDuplications() {
		return duplications;
	}

	/**
	 * @param duplications
	 *            the duplications to set
	 */
	public void setDuplications(String duplications) {
		this.duplications = duplications;
	}

	/**
	 * @return the hostswitches
	 */
	public String getHostswitches() {
		return hostswitches;
	}

	/**
	 * @param hostswitches
	 *            the hostswitches to set
	 */
	public void setHostswitches(String hostswitches) {
		this.hostswitches = hostswitches;
	}

	/**
	 * @return the cospeciationsSingle
	 */
	public String getCospeciationsSingle() {
		return cospeciationsSingle;
	}

	/**
	 * @param cospeciationsSingle
	 *            the cospeciationsSingle to set
	 */
	public void setCospeciationsSingle(String cospeciationsSingle) {
		this.cospeciationsSingle = cospeciationsSingle;
	}

	/**
	 * @return the sortingsSingle
	 */
	public String getSortingsSingle() {
		return sortingsSingle;
	}

	/**
	 * @param sortingsSingle
	 *            the sortingsSingle to set
	 */
	public void setSortingsSingle(String sortingsSingle) {
		this.sortingsSingle = sortingsSingle;
	}

	/**
	 * @return the duplicationsSingle
	 */
	public String getDuplicationsSingle() {
		return duplicationsSingle;
	}

	/**
	 * @param duplicationsSingle
	 *            the duplicationsSingle to set
	 */
	public void setDuplicationsSingle(String duplicationsSingle) {
		this.duplicationsSingle = duplicationsSingle;
	}

	/**
	 * @return the hostswitchesSingle
	 */
	public String getHostswitchesSingle() {
		return hostswitchesSingle;
	}

	/**
	 * @param hostswitchesSingle
	 *            the hostswitchesSingle to set
	 */
	public void setHostswitchesSingle(String hostswitchesSingle) {
		this.hostswitchesSingle = hostswitchesSingle;
	}

	/**
	 * @return the hostMapping
	 */
	public String getHostMapping() {
		return hostMapping;
	}

	/**
	 * @param hostMapping
	 *            the hostMapping to set
	 */
	public void setHostMapping(String hostMapping) {
		this.hostMapping = hostMapping;
	}

}
