/**
 * 
 */
package com.wieseke.cptk.output.viewer;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.common.api.IPositionNode;
import com.wieseke.cptk.common.viewer.CophylogenyViewer;
import com.wieseke.cptk.common.viewer.NodeControl;
import com.wieseke.cptk.output.dao.OutputParasiteNode;
import com.wieseke.cptk.output.preferences.OutputPreferenceConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class OutputParasiteNodeControl extends OutputNodeControl {

	/**
	 * default constructor
	 * 
	 * @param viewer
	 * @param node
	 * @param parent
	 */
	public OutputParasiteNodeControl(CophylogenyViewer viewer,
			IPositionNode node, NodeControl parent) {
		super(viewer, node, parent);
	}

	protected void init() {
		IPreferenceStore store = PlatformUI.getPreferenceStore();
		this.width = store
				.getInt(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEWIDTH);
		this.height = store
				.getInt(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEHEIGHT);
		this.foldSignColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEFOLDSIGNCOLOR));
		this.nodeColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODECOLOR));
		this.nodeHighlightColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEHIGHLIGHTCOLOR));
		this.nodeOutlineColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEOUTLINECOLOR));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.output.viewer.OutputNodeControl#getNode()
	 */

	public OutputParasiteNode getNode() {
		return (OutputParasiteNode) super.getNode();
	}

	public void movePositionDif(int difX, int difY, GC gc, boolean forceDraw) {
		for (int i = 0; i < this.getNode().getEdgePositions().size(); i++) {
			Point point = this.getNode().getEdgePositions().get(i);
			int x = normalize(scale(point.x) + difX);
			int y = normalize(scale(point.y) + difY);
			this.getNode().getEdgePositions().set(i, new Point(x, y));
		}
		super.movePositionDif(difX, difY, gc, forceDraw);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.wieseke.cptk.common.viewer.NodeControl#flipHorizontal(org.eclipse
	 * .swt.graphics.Rectangle)
	 */

	public void flipHorizontal(Rectangle rect) {
		for (int i = 0; i < this.getNode().getEdgePositions().size(); i++) {
			Point point = this.getNode().getEdgePositions().get(i);
			int x = rect.x + (rect.width - point.x) - getNormalizedSize().x;
			int y = point.y;
			this.getNode().getEdgePositions().set(i, new Point(x, y));
		}
		super.flipHorizontal(rect);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.wieseke.cptk.common.viewer.NodeControl#flipVertical(org.eclipse.swt
	 * .graphics.Rectangle)
	 */

	public void flipVertical(Rectangle rect) {
		for (int i = 0; i < this.getNode().getEdgePositions().size(); i++) {
			Point point = this.getNode().getEdgePositions().get(i);
			int x = point.x;
			int y = rect.y + (rect.height - point.y) - getNormalizedSize().y;
			this.getNode().getEdgePositions().set(i, new Point(x, y));
		}
		super.flipVertical(rect);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.wieseke.cptk.common.viewer.NodeControl#rotate(org.eclipse.swt.graphics
	 * .Rectangle)
	 */

	public void rotate(Rectangle rect) {
		for (int i = 0; i < this.getNode().getEdgePositions().size(); i++) {
			Point point = this.getNode().getEdgePositions().get(i);
			int x = point.y + (rect.x - rect.y);
			int y = point.x + (rect.y - rect.x);
			this.getNode().getEdgePositions().set(i, new Point(x, y));
		}
		super.rotate(rect);
	}

}
